#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <math.h>
#include <stdint.h>
#define STATIC static
#include "kcl03.c"

// OS
#if (defined(KWINDOWS))
	#include <windows.h>
#elif (defined(KPOSIX))
	#include <unistd.h>
	#include <sys/mman.h>
#endif

// graphics
#if (defined(KSDL1))
	#include <SDL/SDL.h>
#elif (defined(KSDL2))
	#include <SDL2/SDL.h>
#endif
#if (defined(KWINDOWS))
	#include <io.h>
#endif

#include "kcll00.c"
#include "bla.c"
#include "bla2.c"
#include "chr.c"

KMemPool kmemPool;

enum { R00 = 0, R01, R02, R03, R04, R05, R06, R07 }; // RAX,EAX ` RDI,EDI

typedef struct VInfo_ {
	KInt ofs, typ;
} VInfo;

typedef struct Env_ {
	KLexer lx;
	KVarArea var, vInfo;
	unsigned char *code, *loadRegs, *saveRegs, *exitForce, *exitProg, *brg0, *brg1, *brg2, *runParam;
	KInt qc, ln, timemode, lwait, lwcount, sysLine, sysBreak, sysReg0, sysReg1, codedump, sysRetVal, r04_0;
	KVarArea prog, batch;
	char cmdlin[65536];
	bla_Window *gwin;
	unsigned char gcb[128 * 128 * 2];
	int chrofs_x, chrofs_y, progseln;
	KStringId spCacheKey;	// ЂƂ܂߂ǂ̂ŒGȎ.
	KVarArea spCacheVal;
	KIndexDict0 brg;
	char mode_consonly, mode_escterm, mode_mkexe;
	int escterm_expand_x, escterm_expand_y, escterm_reduce_x, escterm_reduce_y;
	KVarArea mkexe_txt, mkexe_idt, mkexe_lnk, mkexe_dll;
	KIndexDict0 mkexe_vofs;
	KInt mkexe_bss;
} Env;

typedef struct EnvLocal_ {
	KInt progselm;	// ̔ԍ.
	struct EnvLocal_ *envl0;
	KInt qc0, cmplTimeNest, err, ndbg, p316_l;
	KVarArea UsrStk, alias, stkFor, stkIf, stkDo, gotoLink, cmplTime;
	KIndexDict0 label;
	KInt connect_c, connect_x, connect_y;
} EnvLocal;

Env *env;
EnvLocal *envl = 0;

typedef struct Prog_ {
	KSrc src;
	KVarArea *lc;
	KInt dtrp;
} Prog;

Prog *Prog_init(Prog *w, KMemPool *mp)
{
	w = KMemPool_allocP0(mp, sizeof (Prog), w);
	KSrc_init(&w->src, mp);
	w->lc = 0;
	w->dtrp = 0;
	return w;
}

void *Prog_dest(Prog *w)
{
	KSrc_dest(&w->src);
	return w;
}

void Prog_destFr(Prog *w)
{
	KMemPool_free(w->src.ln.va.mp, sizeof (Prog), Prog_dest(w));
}

Prog *prog_getProg(KInt psel)
{
	if (psel < 0) return 0;
	if (KVarArea_sizPtr(&env->prog) <= psel) return 0;
	if (KVarArea_getPtrIdx(&env->prog, psel) == 0) return 0;
	return (Prog *) KVarArea_getPtrIdx(&env->prog, psel);
}

void prog_delLinCnt(Prog *pp)
{
	if (pp->lc != 0) {
		KVarArea_destFr(pp->lc);
		pp->lc = 0;
	}
}

STATIC KInt getVarOfs(KInt i)
{
	if (env->mode_mkexe == 0)
		return ((VInfo *) &env->vInfo.p[i * sizeof (VInfo)])->ofs;
	else {
		KInt *p = (KInt *) KIndexDict0_search1(&env->mkexe_vofs, &i);
		if (p == 0) {
			p = KMemPool_alloc(&kmemPool, 2 * sizeof (KInt));
			p[0] = i;
			env->mkexe_bss -= sizeof (KInt);
			p[1] = env->mkexe_bss;
			KIndexDict0_ins(&env->mkexe_vofs, p);
		}
		return p[1];
	}
}

inline STATIC KInt *getVarPtr(KInt i)
{
	return (KInt *) &env->var.p[/* getVarOfs(i) */ ((VInfo *) &env->vInfo.p[i * sizeof (VInfo)])->ofs];
}

KInt testConstInt(KInt i)
{
	KStringId_Elmt *e = KStringId_getStr(env->lx.sid, i);
	if (e->l >= 1 && '0' <= e->p[0] && e->p[0] <= '9') return 1;
	if (e->l >= 2 && e->p[0] == '_' && '0' <= e->p[1] && e->p[1] <= '9') return 1;
	return 0;
}

KInt getConstInt(KInt i)
{
	KStringId_Elmt *e = KStringId_getStr(env->lx.sid, i);
	if (e->p[0] == '_') return - kstrtoInt(e->l - 1, &e->p[1], 0, 10);
	return kstrtoInt(e->l, e->p, 0, 0);
}

KInt labelId(KInt i)
{
	if (testConstInt(i)) return getConstInt(i);
	return ~i;
}

void put32(unsigned char *p, KInt i)
{
	p[0] =  i        & 0xff;
	p[1] = (i >>  8) & 0xff;
	p[2] = (i >> 16) & 0xff;
	p[3] = (i >> 24) & 0xff;
}

void testKeyBuf()
{
	static int i = 0;
	while (i != env->gwin->kbw) {
		if ((env->gwin->keybuf[i] & 0xffff) == bla_KEY_HOME && (env->gwin->keybuf[i] & 0x01010000) != 0)
			env->sysBreak |= 2;
		i = (i + 1) & (bla_KEYBUFSIZ - 1);
	}
	if ((env->sysBreak & 2) != 0)
		i = env->gwin->kbr = env->gwin->kbw = 0;
}

KInt einkey()
{
	KInt i = bla_inkey(env->gwin) & ~ bla_KEY_LOCKS;
	if (i == (bla_KEY_END | 0x00010000) || i == (bla_KEY_END | 0x01000000))
		((void (*)()) env->exitForce)();
	return i;
}

bla_Window *es_openWin(int x, int y, const char *t, int autoClose)
{
	bla_Window *w;
	if (env->mode_consonly == 0)
		w = bla_openWin(x, y, t, autoClose);
	else
		w = bla_openVirWin(x, y, t, autoClose);
	return w;
}

void es_flushAll(bla_Window *w)
{
	if (env->mode_consonly == 0)
		bla_flushAll(w);
	else {
		if (w->phase == 1) {
			w->lastFlush = clock();
		}
	}
	if (env->mode_escterm != 0)
		bla_escTerm(w, env->mode_escterm, env->escterm_expand_x, env->escterm_expand_y, env->escterm_reduce_x, env->escterm_reduce_y);
}

void es_closeWin(bla_Window *w)
{
	if (env->mode_consonly == 0)
		bla_closeWin(w);
	else
		bla_closeVirWin(w);
}

STATIC void es_leapFlushAll(bla_Window *w, unsigned int t)
{
	if (t == 0 || (double) (clock() - w->lastFlush) * (1000.0 / (double) CLOCKS_PER_SEC) >= (double) t)
		es_flushAll(w);
}

#if 0

#define DLL_MSVCRT					0x00010000
#define	DLL___GETMAINARGS			DLL_MSVCRT + 0x0000
#define	DLL_PRINTF					DLL_MSVCRT + 0x0001
#define	DLL_STRTOL					DLL_MSVCRT + 0x0002

#elif 1

#define DLL_MSVCRT					0x24
#define	DLL___GETMAINARGS			DLL_MSVCRT + 4
#define	DLL_PRINTF					DLL_MSVCRT + 0
#define	DLL_STRTOL					DLL_MSVCRT + 8

#endif

char *printf_sub(KInt l, const char *s)
{
	char *t = KMemPool_alloc(&kmemPool, l + 1);
	KInt i, j;
	for (i = j = 0; i < l; i++) {
		if (s[i] != '\\')
			t[j++] = s[i];
		else {
			i++;
			if (s[i] == 'n')
				t[j++] = '\n';
			else if (s[i] == 't')
				t[j++] = '\t';
			else if (s[i] == 'r')
				t[j++] = '\r';
			else
				t[j++] = s[i];	// \xƂ肽.
		}
	}
	t[j] = 0;
	return t;
}

void sub_printStr(KInt l, const char *s)	{ char *t = printf_sub(l, s); fputs(t, stdout); KMemPool_free(&kmemPool, l + 1, t); }
void sub_printLf()							{ printf("\n"); }
void sub_printSpc()							{ printf(" "); }
void sub_exit()								{ exit(0); }
void sub_timemode(KInt i)					{ env->timemode = i; }
void sub_lwait(KInt i)						{ env->lwait = i; }
void sub_codedump(KInt i)					{ env->codedump = i; }
void sub_setpix(KInt x, KInt y, KInt c)		{ bla_setPixEx(env->gwin, x, y, bla_xcol(c)); }
void sub_input(KInt *p)						{ int i; scanf("%d", &i); *p = i; }
void sub_rand(KInt *p)						{ *p = kxorshift32(); }
void sub_gcls(KInt c);
void sub_openwin(KInt sx, KInt sy)			{ es_flushAll(env->gwin); bla_waitEx(100); es_closeWin(env->gwin); env->gwin = es_openWin(sx, sy, "graphics", 0); sub_gcls(0); }
void sub_flushwin()							{ es_flushAll(env->gwin); }
void sub_leapflushwin(KInt msec)			{ es_leapFlushAll(env->gwin, msec); }
void sub_ewait(KInt msec)					{ es_flushAll(env->gwin); bla_waitEx(msec); }
void sub_einkey(KInt *p)					{ *p = einkey(); }
void sub_getchr(KInt x, KInt y, KInt *p)	{ *p = env->gcb[y * 256 + x * 2 + 0]; }
void sub_getcol(KInt x, KInt y, KInt *p)	{ *p = env->gcb[y * 256 + x * 2 + 1]; }
void sub_chrofs(KInt x, KInt y)				{ env->chrofs_x = x; env->chrofs_y = y; }
void sub_gprinti(KInt x, KInt y, KInt w, KInt i, KInt c, KInt b) { char t[200]; sprintf(t, "%*d", (int) w, (int) i); bla_drawStrEx(env->gwin, x, y, bla_xcol(c), bla_xcol(b), t); }
void sub_gprints(KInt x, KInt y, KInt l, const char *s, KInt c, KInt b) { char t[200]; memcpy(t, s, l); t[l] = 0; bla_drawStrEx(env->gwin, x, y, bla_xcol(c), bla_xcol(b), t); }
void sub_fillrect(KInt xs, KInt ys, KInt x0, KInt y0, int c) { bla_fillRect(env->gwin, xs, ys, x0, y0, bla_xcol(c)); }
KInt sub_rgbcol(KInt r, KInt g, KInt b)		{ return b | g << 8 | r << 16 | 0x1000000; }
void sub_aryintdel(KVarArea *p)				{ KVarArea_destFr(p); }
KInt sub_ff16sin(KInt x)					{ return (KInt) (sin(x * (2 * 3.14159265358979323 / 65536)) * 65536); }
KInt sub_ff16cos(KInt x)					{ return (KInt) (cos(x * (2 * 3.14159265358979323 / 65536)) * 65536); }
void sub_end()								{ *getVarPtr(env->sysLine) = -2; /* longjmp(env->jb, 1); */ }
void sub_progseln(KInt *p)					{ *p = env->progseln; }
void sub_errstop()							{ fprintf(stderr, "\nexectime error\n"); }
KInt *sub_getPR04_0()						{ return &env->r04_0; }
void sub_fillrectex(KInt sx, KInt sy, KInt x0, KInt y0, KInt c)	{ bla_fillRectEx(env->gwin, sx, sy, x0, y0, bla_xcol(c)); }
void sub_drawrectex(KInt sx, KInt sy, KInt x0, KInt y0, KInt c)	{ bla_drawRectEx(env->gwin, sx, sy, x0, y0, bla_xcol(c)); }
void sub_drawlineex(KInt x0, KInt y0, KInt x1, KInt y1, KInt c)	{ bla_drawLineEx(env->gwin, x0, y0, x1, y1, bla_xcol(c)); }
void sub_fillovalex(KInt sx, KInt sy, KInt x0, KInt y0, KInt c)	{ bla_fillOvalEx(env->gwin, sx, sy, x0, y0, bla_xcol(c)); }
void sub_drawovalex(KInt sx, KInt sy, KInt x0, KInt y0, KInt c)	{ bla_drawOvalEx(env->gwin, sx, sy, x0, y0, bla_xcol(c)); }
void sub_fillex(KInt x, KInt y, KInt c)		{ bla_fillEx(env->gwin, x, y, bla_xcol(c)); }
KInt sub_ff16sqrt(KInt x)					{ return (KInt) (sqrt(x * (1.0 / 65536.0)) * 65536); }
void sub_drawline(KInt x0, KInt y0, KInt x1, KInt y1, KInt c)	{ bla_drawLine(env->gwin, x0, y0, x1, y1, bla_xcol(c)); }
void sub_clrkeybuf()						{ while (einkey() != 0); }
void sub_kwait(KInt i)						{ es_flushAll(env->gwin); do { bla_waitEx(100); } while (einkey() != i); }
void sub_getpix(KInt x, KInt y, KInt *c)	{ *c = bla_getPix(env->gwin, x, y); }
void sub_setgmode(KInt m)					{ bla_setMode(env->gwin, m); }

void sub_gcls(KInt c)
{
	int i, *p = env->gwin->buf, xys = env->gwin->xsiz * env->gwin->ysiz;
	for (i = 0; i < 128 * 128; i++) {
		env->gcb[i * 2 + 0] = 0;
		env->gcb[i * 2 + 1] = c;
	}
	c = bla_xcol(c);
	for (i = 0; i < xys; i++)
		p[i] = c;
	envl->connect_x = -0x7fffffff;
}

#define PF KPRINTF_FORMAT

void sub_print(KInt i)
{
	printf(PF("%d", "%I64d", "%ld"), i);
}

void sub_progsel(KInt i)
{
	while (KVarArea_sizPtr(&env->prog) <= i)
		KVarArea_addPtr(&env->prog, 0);
	if (KVarArea_getPtrIdx(&env->prog, i) == 0)
		KVarArea_setPtrIdx(&env->prog, i, Prog_init(0, &kmemPool));
	env->progseln = i;
}

void sub_list(KInt l0, KInt l1)
{
	l0 = 0;
	l1 = 0x7fffffff;
	Prog *pp = prog_getProg(env->progseln);
	if (pp == 0) return;
	KIndexS *pn = &pp->src.ln;
	KInt cr, i = KIndexS_search(pn, &l0, &cr);
	while (i < pn->va.s) {
		KSrcLine *pl = *(KSrcLine **) &pn->va.p[i];
		if (pl->ln > l1) break;
		printf("%4d %.*s\n", (int) pl->ln, (int) pl->src.s, pl->src.p);
		i += sizeof (void *);
	}
}

KInt sub_initstack(KInt s)
{
	if (s > 0) {
		KVarArea_reserve(&envl->UsrStk, s * sizeof (KInt));
		envl->UsrStk.s = 0;
	}
	return (KInt) envl->UsrStk.p;
}

void sub_echr(KInt x, KInt y, KInt ch, KInt c, KInt b)
{
	bla_ECWin ecw;
	ecw.win = env->gwin;
	ecw.cb = env->gcb;
	ecw.font = chr;
	bla_echr(&ecw, x + env->chrofs_x, y + env->chrofs_y, ch, c, b);
}

void sub_chrbox(KInt sx, KInt sy, KInt x, KInt y, KInt ch, KInt c, KInt b)
{
	KInt i, j;
	for (j = 0; j < sy; j++) {
		for (i = 0; i < sx; i++)
			sub_echr(x + i, y + j, ch, c, b);
	}
}

void sub_scroll(KInt d)
{
	bla_ECWin ecw;
	ecw.win = env->gwin;
	ecw.cb = env->gcb;
	ecw.font = chr;
	bla_scroll1(&ecw, d);
}

void sub_proglist()
{
	KInt i, i1 = KVarArea_sizPtr(&env->prog);
	for (i = 0; i < i1; i++) {
		KSrc *pr = (KSrc *) KVarArea_getPtrIdx(&env->prog, i);
		if (pr != 0)
			printf("  #%02d [%d]\n", (int) i, (int) KVarArea_sizPtr(&pr->ln.va));
	}
	printf ("  progseln=%02d\n", (int) env->progseln);
}

void sub_new(KInt psel)
{
	Prog *pp = prog_getProg(psel);
	if (pp != 0) {
		KSrc_delAll(&pp->src);
		prog_delLinCnt(pp);
	}
}

KInt sub_f(KInt k)
{
	if (k == 0) return kxorshift32();
	if (k == 1) { int i; scanf("%d", &i); return i; }
	if (k == 2) return einkey();
	return 0;
}

void sub_aryintnew(KVarArea **p, KInt s, KInt u)
{
	*p = KMemPool_alloc(&kmemPool, sizeof (KVarArea));
	KVarArea_init(*p, &kmemPool, s * u);
}

void sub_prognew(KInt *p)
{
	KInt i, i1 = KVarArea_sizPtr(&env->prog);
	for (i = 0; i < i1; i++) {
		if (KVarArea_getPtrIdx(&env->prog, i) == 0) break;
	}
	if (i >= i1)
		KVarArea_addPtr(&env->prog, 0);
	KVarArea_setPtrIdx(&env->prog, i, Prog_init(0, &kmemPool));
	*p = i;
}

void sub_progdel(KInt i)
{
	if (i == env->progseln) return;
	if (prog_getProg(i) == 0) return;
	sub_new(i);
	Prog_destFr((Prog *) KVarArea_getPtrIdx(&env->prog, i));
	KVarArea_setPtrIdx(&env->prog, i, 0);
}

void sub_linecounter(KInt psel)
{
	Prog *pp = prog_getProg(psel);
	if (pp == 0) return;
	if (pp->lc == 0) return;
	KInt *cc = (KInt *) pp->lc->p, i, i1 = KVarArea_sizPtr(&pp->src.ln.va);
	for (i = 0; i < i1; i++) {
		KSrcLine *pl = (KSrcLine *) KVarArea_getPtrIdx(&pp->src.ln.va, i);
		printf(PF("%08x%08x", "%016I64x%016I64x", "%016lx%016lx") "  %4d %.*s\n", cc[i * 2 + 1], cc[i * 2 + 0], (int) pl->ln, (int) pl->src.s, pl->src.p);
	}
}

void sub_renum(KInt psel, KInt n0)
{
	KInt st = 10;
	if (n0 < 10)
		st = 1;
	Prog *pp = prog_getProg(psel);
	if (pp == 0) return;
	KInt i, i1 = KVarArea_sizPtr(&pp->src.ln.va);
	for (i = 0; i < i1; i++) {
		KSrcLine *pl = (KSrcLine *) KVarArea_getPtrIdx(&pp->src.ln.va, i);
		pl->ln = i * st + n0;
	}
}

void cmdlineLoop(char flg);

void sub_pause()
{
	KInt i = *getVarPtr(env->sysLine);
	if (i >= 0)
		printf("\npause in %d\n", (int) i);
	else
		printf("\npause\n");
	cmdlineLoop(1);
}

void sub_dtrp(KInt psel, KInt i)
{
	Prog *pp = prog_getProg(psel);
	if (pp != 0) {
		if (testConstInt(i) != 0 && getConstInt(i) == 0)
			pp->dtrp = 0;
		else
			pp->dtrp = labelId(i);
	}
}

static KInt sub_lwait__i;

void sub_lwait_()
{
	env->lwcount += env->lwait;
	if (env->lwcount >= 10000000)
		sub_lwait__i = 0;
	sub_lwait__i--;
	if (sub_lwait__i < 0) {
		sub_lwait__i = 255;
		testKeyBuf();
//		if ((env->sysBreak & 1) != 0)
//			/* longjmp(env->jb, 1) */;
		if ((env->sysBreak & 2) != 0)
			sub_pause();
	}
	while (env->lwcount >= 10000000) {
		env->lwcount -= 10000000;
		es_leapFlushAll(env->gwin, 100);
		bla_wait(10);
	}
}

void sub_resetlc(KInt psel)
{
	Prog *pp = prog_getProg(psel);
	if (pp != 0 && pp->lc != 0)
		memset(pp->lc->p, 0, pp->lc->s);	// resetLineExecCount.
}

void sub_bitblt(KInt sx, KInt sy, KInt x0, KInt y0, KVarArea *p, KInt i)
{
	bla_Window *w = env->gwin;
	if (0 <= x0 && x0 + sx <= w->xsiz && 0 <= y0 && y0 + sy <= w->ysiz && p->s >= i + sx * sy * (KInt) sizeof (KInt)) {
		KInt x, y, *q = ((KInt *) p->p) + i;
		int *b = &w->buf[y0 * w->xsiz + x0];
		for (y = 0; y < sy; y++) {
			for (x = 0; x < sx; x++)
				b[x] = q[x];
			q += sx;
			b += w->xsiz;
		}
	}
}

void sub_savewin(KInt l, const char *s, KInt i, KInt w, KInt h, KInt x0, KInt y0)
{
	char *t = KMemPool_alloc(&kmemPool, l + 1);
	char *u = KMemPool_alloc(&kmemPool, l + 100);
	memcpy(t, s, l);
	t[l] = 0;
	sprintf(u, t, i);
	bla_saveBitmapToFile(env->gwin, u, w, h, x0, y0);
	KMemPool_free(&kmemPool,l + 1, t);
	KMemPool_free(&kmemPool,l + 100, u);
}

void sub_connect(KInt x, KInt y, KInt c)
{
	if (c == -1)
		c = envl->connect_c;
	else
		envl->connect_c = c = bla_xcol(c);
	if (x != -0x7fffffff) {
		if (envl->connect_x == -0x7fffffff) {
			envl->connect_x = x;
			envl->connect_y = y;
		}
		bla_drawLineEx(env->gwin, envl->connect_x, envl->connect_y, x, y, c);
	}
	envl->connect_x = x;
	envl->connect_y = y;
}

void sub_fillovalrex(KInt x0, KInt y0, KInt c, KInt r, KInt r1)
{
	if (r1 == -1)
		r1 = r;
	bla_fillOvalEx(env->gwin, r << 1, r1 << 1, x0 - r, y0 - r1, bla_xcol(c));
}

void sub_drawovalrex(KInt x0, KInt y0, KInt c, KInt r, KInt r1)
{
	if (r1 == -1)
		r1 = r;
	bla_drawOvalEx(env->gwin, r << 1, r1 << 1, x0 - r, y0 - r1, bla_xcol(c));
}

void sub_bltbit(KInt sx, KInt sy, KInt x0, KInt y0, KVarArea *p, KInt i)
{
	bla_Window *w = env->gwin;
	if (0 <= x0 && x0 + sx <= w->xsiz && 0 <= y0 && y0 + sy <= w->ysiz && p->s >= i + sx * sy * (KInt) sizeof (KInt)) {
		KInt x, y, *q = ((KInt *) p->p) + i;
		int *b = &w->buf[y0 * w->xsiz + x0];
		for (y = 0; y < sy; y++) {
			for (x = 0; x < sx; x++)
				q[x] = b[x];
			q += sx;
			b += w->xsiz;
		}
	}
}

void sub_bitbltx(KInt sx, KInt sy, KInt x0, KInt y0, KVarArea *p, KInt i, KInt rx, KInt ry)
{
	if (ry == 0)
		ry = rx;
	KInt qx = - rx, qy = - ry;
	if (p->s >= i + sx * sy * (KInt) sizeof (KInt)) {
		KInt x, y, *q = ((KInt *) p->p) + i;
		for (y = 0; y < sy; y++) {
			if (ry <= -1 && y % qy != 0) goto skip;
			for (x = 0; x < sx; x++) {
				KInt c = q[x];
				if (rx <= -1 && x % qx != 0) continue;
				if (rx >= 1 && ry >= 1)
					bla_fillRectEx(env->gwin, rx, ry, x0 + x * rx, y0 + y * ry, c);
				if (rx <= -1 && ry <= -1)
					bla_setPixEx(env->gwin, x0 + x / qx, y0 + y / qy, c);
			}
skip:
			q += sx;
		}
	}
}

void sub_printf(KInt l, char *s, KInt i0, KInt i1, KInt i2, KInt i3, KInt i4, KInt i5, KInt i6, KInt i7)
{
	char *t = printf_sub(l, s);
	printf(t, i0, i1, i2, i3, i4, i5, i6, i7);
	KMemPool_free(&kmemPool, l + 1, t);
}

void sub_printff16(KInt l, char *s, KInt f0, KInt f1, KInt f2, KInt f3, KInt f4, KInt f5, KInt f6, KInt f7)
{
	char *t = printf_sub(l, s);
	printf(t, f0 * (1.0 / 65536), f1 * (1.0 / 65536), f2 * (1.0 / 65536), f3 * (1.0 / 65536),
		f4 * (1.0 / 65536), f5 * (1.0 / 65536), f6 * (1.0 / 65536), f7 * (1.0 / 65536));
	KMemPool_free(&kmemPool, l + 1, t);
}

void sub_escterm(KInt i, KInt ex, KInt ey, KInt rx, KInt ry)
{
	bla_initEscTerm();
	env->mode_escterm = i;
	if (ex < 0) {
		rx = - ex;
		ex = 1;
	}
	if (ey < 0) {
		ry = - ey;
		ey = 1;
	}
	env->escterm_expand_x = ex;
	env->escterm_expand_y = ey;
	env->escterm_reduce_x = rx;
	env->escterm_reduce_y = ry;
}

void sub_loadwin(KInt l, const char *s, KInt i, KInt x0, KInt y0)
{
	char *t = KMemPool_alloc(&kmemPool, l + 1);
	char *u = KMemPool_alloc(&kmemPool, l + 100);
	memcpy(t, s, l);
	t[l] = 0;
	sprintf(u, t, i);
	bla_loadBitmapFromFile(env->gwin, u, x0, y0);
	KMemPool_free(&kmemPool,l + 1, t);
	KMemPool_free(&kmemPool,l + 100, u);
}

void sub_aryintdump6x(KVarArea *a, KInt i)
{
	KInt ii, ii1 = KVarArea_sizInt(a);
	KStringId_Elmt *e = KStringId_getStr(env->lx.sid, i);
	for (ii = 0; ii < ii1; ii++) {
		if (ii % 8 == 0)
			printf("ARY INT %.*s[%d...]={", (int) e->l, e->p, (int) ii);
		else
			printf(",");
		printf("0X%06X", (unsigned int) KVarArea_getIntIdx(a, ii));
		if (ii % 8 == 7 || ii + 1 == ii1)
			printf("};\n");
	}
}

void sub_getarg(KInt *p0, KInt *p1, KInt *p2, KInt *p3)
{
	unsigned char *p = env->runParam + 4; // "RUN "
	while (0 < *p && *p <= ' ') 	p++;
	if (p0 != 0 && *p != 0)
		*p0 = strtol((char *) p, (char **) &p, 0);
	while (0 < *p && *p <= ' ') p++;
	if (*p == ',') { p++; while (0 < *p && *p <= ' ') p++; }
	if (p1 != 0 && *p != 0)
		*p1 = strtol((char *) p, (char **) &p, 0);
	while (0 < *p && *p <= ' ') p++;
	if (*p == ',') { p++; while (0 < *p && *p <= ' ') p++; }
	if (p2 != 0 && *p != 0)
		*p2 = strtol((char *) p, (char **) &p, 0);
	while (0 < *p && *p <= ' ') p++;
	if (*p == ',') { p++; while (0 < *p && *p <= ' ') p++; }
	if (p3 != 0 && *p != 0)
		*p3 = strtol((char *) p, (char **) &p, 0);
}

int sub_run(KInt psel);

typedef struct StkFor_ {
	KInt qc, c, c1, d, ne;
} StkFor;

typedef struct StkIf_ {
	KInt qc0, qc1;
} StkIf;

inline STATIC KInt testReg(KInt i)
{
	return env->sysReg0 <= i && i < env->sysReg1;
}

void cb_kstringId(KStringId *sid, KInt i)
{
	KVarArea_addInt(&env->vInfo, env->var.s);
	KVarArea_addInt(&env->vInfo, 1);	// type:KInt.
	KStringId_Elmt *e = KStringId_getStr(sid, i);
#if (defined(KUSE64))
	if (e->l == 6 && strncmp(e->p, "RFLAGS", 6) == 0)
#elif (defined(KUSE32))
	if (e->l == 6 && strncmp(e->p, "EFLAGS", 6) == 0)
#endif
		KVarArea_addInt(&env->var, 0x202);
	else
		KVarArea_addInt(&env->var, getConstInt(i));
	if (envl != 0)
		KVarArea_addInt(&envl->alias, i);
}

KInt selectReg(KInt i, KInt r0)
{
	if (env->sysReg0 <= i && i < env->sysReg1)
		return i - env->sysReg0;
	return r0;
}

KInt storeReg(KInt reg, KInt i, unsigned char *q)
{
	if (i == env->sysReg0 + reg)
		return 0;
	if (env->sysReg0 <= i && i < env->sysReg1)
		return kputBin86(q, "%rx1_89_%mrm", 3, (int) reg, (int) (i - env->sysReg0));
//	return kputBin86(q, "%rx1_89_%mrm_%i32", 2, (int) reg, R05, getVarOfs(i));
	return kputBin86(q, "%rx1_89_%m0", (int) reg, R05, getVarOfs(i));
}

KInt loadRegImm(KInt reg, unsigned char *q, KInt i)
{
	KInt c = 0;
	if (i == 0) {
		if (reg < 8)
			c += kputBin86(&q[c], "31_%mrm", 3, (int) reg, (int) reg);
		else
			c += kputBin86(&q[c], "%rx1_31_%mrm", 3, (int) reg, (int) reg);
	} else if (i == -1) {
		c += kputBin86(&q[c], "%rx1_83_%mrm_ff", 3, 1, (int) reg);
#if (defined(KUSE64))
	} else if (0 < i && i <= 0xffffffffLL) {
		if (reg < 8)
			c += kputBin86(&q[c], "%cb_%i32", 0xb8, (int) reg, i);
		else
			c += kputBin86(&q[c], "%rx0_%cb_%i32", 0xb8, (int) reg, i);
		} else if (-0x80000000LL < i && i < 0) {
			c += kputBin86(&q[c], "%rx1_c7_%mrm_%i32", 3, 0, (int) reg, i);
#endif
	} else
		c += kputBin86(&q[c], "%rx1_%cb_%iUS", 0xb8, (int) reg, i);
	return c;
}

KInt loadReg0(KInt reg, KInt i, unsigned char *q)
{
	if (i == env->sysReg0 + reg)
		return 0;
	if (env->sysReg0 <= i && i < env->sysReg1)
		return kputBin86(q, "%rx1_89_%mrm", 3, (int) (i - env->sysReg0), (int) reg);
	if (testConstInt(i))
		return loadRegImm(reg, q, getConstInt(i));
//	return kputBin86(q, "%rx1_8b_%mrm_%i32", 2, (int) reg, R05, getVarOfs(i));
	return kputBin86(q, "%rx1_8b_%m0", (int) reg, R05, getVarOfs(i));
}

#if (defined(KUSE64))

void *bridge(void *f)
// f0->f1ւ̕ϊp̃f[^\pӂȂŁAbrg116oCg܂pĂ.
{
	unsigned char *p = (unsigned char *) KIndexDict0_search1(&env->brg, &f);
	if (p == 0) {
		p = env->brg1;
		env->brg1 += 16;
		if (env->brg1 > env->brg2)
			kerrorExit("bridge: internal error");
		kputBin86(p, "%rx1_ff_25_01_00_00_00_00_%i64", f);	// JMP NEAR QWORD [RIP+1] / ALIGNB 8 (==RESB 1) / DQ f
		p += 8; // p邽߂̃|C^炵.
		KIndexDict0_ins(&env->brg, p);
	}
	return p - 8; // p邽߂̃|C^炵␳.
}

#define BRG(f)	bridge(f)

#elif (defined(KUSE32))

#define BRG(f)	f

#endif

#define NOPARAM		0x8000005a

typedef struct OpCall_ {
	KInt *pqc, *pos;
	unsigned char *code;
	KLexer_Elmt *le;
} OpCall;

#define PC	KPhraseCmp_cmpPr

void OpCall_put(OpCall *w, const char *s,...);

KInt stopSub(unsigned char *cod)
{
	KInt qc = 0;
	qc += kputBin86(&cod[qc], "b8_%i32   %rx1_89_%m0", (KInt) env->ln, R00, R05, (KInt) getVarOfs(env->sysLine));
	qc += kputBin86(&cod[qc], "e8_%r32   %rx1_8b_%mrm", (KInt) BRG(sub_getPR04_0), 0, R04, R00);
	qc += kputBin86(&cod[qc], "e9_%r32", (KInt) env->exitProg);
	return qc;
}

KInt execTimeError(unsigned char *q, int jcc)
{
	KInt c = 0;
	if (env->lwait >= 0 && envl->ndbg == 0) {
		#if (defined(KUSE64))
			c += kputBin86(&q[c], "%i8_%i8", jcc, 10 + 25);
		#elif (defined(KUSE32))
			c += kputBin86(&q[c], "%i8_%i8", jcc, 10 + 23);
		#endif
		c += kputBin86(&q[c], "e8_%r32", (KInt) env->saveRegs);
		c += kputBin86(&q[c], "e8_%r32", (KInt) BRG(sub_errstop));
		c += stopSub(&q[c]);
	}
	return c;
}

KInt loadReg(KInt reg, unsigned char *q, KStringId *sid, KLexer_Elmt *le, KInt *pos)
{
	static KPhraseCmp *phr;
	if (phr == 0) {
		phr = KPhraseCmp_init(0, &kmemPool, 32, 100, env->lx.sid, env->lx.chrTyp, 0);
		KPhraseCmp_cmp(phr, 0, "@ ( ) + - * / % << >> RAND INPUT EINKEY | & ^ *:: >> [ ] FF16SIN FF16COS FF16SQRT :", 0, 0, 0, 0);
	}
	KInt i, i0 = pos[0], i1 = pos[1], c = 0, j, k;
	char f = 0;
	for (i = i0; i < i1; i++) {
		if (le[i].i == phr->code[0])
			continue;
		if (le[i].i == phr->code[1])
			continue;
		if (f != 0) {
			for (j = 0; j < 8; j++) {
				static int tbl[8] = { 2, 12, -1, -1, 13, 3, 14, -1 };
				if (tbl[j] < 0) continue;
				if (le[i].i == phr->code[tbl[j]] && i + 1 < i1) {
					if (testConstInt(le[i + 1].i)) {
						k = getConstInt(le[i + 1].i);
						if (k == 0 && (j == 0 || j == 5 || j == 1 || j == 6)) { i++; break; }
						if (k == -1 && j == 4) { i++; break; }
						if (-128 <= k && k <= 127) {
							c += kputBin86(&q[c], "%rx1_83_%mrm_%i8", 3, (int) j, (int) reg, k);
#if (defined(KUSE64))
						} else if (k < -0x80000000LL || 0x7fffffffLL < k) {
							kerrorExit("loadReg: mikan");
#endif
						} else
							c += kputBin86(&q[c], "%rx1_81_%mrm_%i32", 3, (int) j, (int) reg, k);
					} else if (testReg(le[i + 1].i))
						c += kputBin86(&q[c], "%rx1_%i8_%mrm", j << 3 | 3, 3, (int) reg, (int) (le[i + 1].i - env->sysReg0));
					else
						c += kputBin86(&q[c], "%rx1_%i8_%m0", j << 3 | 3, (int) reg, 5, getVarOfs(le[i + 1].i));
					i++;
					if (j == 0 || j == 5)
						c += execTimeError(&q[c], 0x71);
					break;
				}
			}
			if (j < 8) continue;
			if (le[i].i == phr->code[4] && i + 1 < i1) {	// Z.
				if (testConstInt(le[i + 1].i)) {
					j = getConstInt(le[i + 1].i);
					if (j == 0)
						c += kputBin86(&q[c], "%rx1_31_%mrm", 3, (int) reg, (int) reg);
					else if (j == 1)
						;
					else if ((j & (j - 1)) == 0 && !(env->lwait >= 0 && envl->ndbg == 0))
						c += kputBin86(&q[c], "%rx1_c1_%mrm_%i8", 3, 4, (int) reg, kpopCount32(j - 1));
					else {
						c += kputBin86(&q[c], "%rx1_69_%mrm_%i32", 3, (int) reg, (int) reg, j);
						c += execTimeError(&q[c], 0x71);
					}
				} else {
					if (testReg(le[i + 1].i))
						c += kputBin86(&q[c], "%rx1_0f_af_%mrm", 3, (int) reg, (int) (le[i + 1].i - env->sysReg0));
					else
						c += kputBin86(&q[c], "%rx1_0f_af_%m0", (int) reg, 5, getVarOfs(le[i + 1].i));
					c += execTimeError(&q[c], 0x71);
				}
				i++;
				continue;
			}
			if (le[i].i == phr->code[5] && i + 1 < i1) {	// Z.
				KInt reg1 = selectReg(le[i + 1].i, R01);
				c += loadReg0(reg1, le[i + 1].i, &q[c]);
				if (reg != R00)
					c += kputBin86(&q[c], "%rx1_89_%mrm", 3, (int) reg, R00);
				if (env->lwait >= 0 && envl->ndbg == 0) {
					c += kputBin86(&q[c], "%rx1_85_%mrm", 3, (int) reg1, (int) reg1);
					c += execTimeError(&q[c], 0x75);
				}
				c += kputBin86(&q[c], "%rx1_99  %rx1_f7_%mrm", 3, 7, (int) reg1);
				if (reg != R00)
					c += kputBin86(&q[c], "%rx1_89_%mrm", 3, R00, (int) reg);
				i++;
				continue;
			}
			if (le[i].i == phr->code[6] && i + 1 < i1) {	// ].
				KInt reg1 = selectReg(le[i + 1].i, R01);
				c += loadReg0(reg1, le[i + 1].i, &q[c]);
				if (reg != R00)
					c += kputBin86(&q[c], "%rx1_89_%mrm", 3, (int) reg, R00);
				if (env->lwait >= 0 && envl->ndbg == 0) {
					c += kputBin86(&q[c], "%rx1_85_%mrm", 3, (int) reg1, (int) reg1);
					c += execTimeError(&q[c], 0x75);
				}
				c += kputBin86(&q[c], "%rx1_99  %rx1_f7_%mrm", 3, 7, (int) reg1);
				if (reg != R02)
					c += kputBin86(&q[c], "%rx1_89_%mrm", 3, R02, (int) reg);
				i++;
				continue;
			}
			if (le[i].i == phr->code[7] && i + 1 < i1 && testConstInt(le[i + 1].i) != 0) {	// <<.
				KInt j = getConstInt(le[i + 1].i);
				if (j != 0)
					c += kputBin86(&q[c], "%rx1_c1_%mrm_%i8", 3, 4, (int) reg, (int) j);
				i++;
				continue;
			}
			if (le[i].i == phr->code[8] && i + 1 < i1 && testConstInt(le[i + 1].i) != 0) {	// >>.
				KInt j = getConstInt(le[i + 1].i);
				if (j != 0)
					c += kputBin86(&q[c], "%rx1_c1_%mrm_%i8", 3, 7, (int) reg, (int) j);
				i++;
				continue;
			}
			if (le[i].i == phr->code[15] && i + 3 < i1 && le[i + 2].i == phr->code[16] && testConstInt(le[i + 3].i) != 0) {	// *:: >>
				KInt reg1 = selectReg(le[i + 1].i, R02);
				c += loadReg0(reg1, le[i + 1].i, &q[c]);
				if (reg != R00)
					c += kputBin86(&q[c], "%rx1_89_%mrm", 3, (int) reg, R00);
				c += kputBin86(&q[c], "%rx1_f7_%mrm", 3, 5, (int) reg1);
				KInt j = getConstInt(le[i + 3].i);
				if (j != 0)
					c += kputBin86(&q[c], "%rx1_0f_ac_d0_%i8", getConstInt(le[i + 3].i));
				if (reg != R00)
					c += kputBin86(&q[c], "%rx1_89_%mrm", 3, R00, (int) reg);
				i += 3;
				continue;
			}
err:
			fprintf(stderr, "expr warning [%d(i=%d)] : ", (int) env->ln, (int) (i - i0));
			for (i = i0; i < i1; i++) {
				KStringId_Elmt *ke = KStringId_getStr(sid, le[i].i);
				fprintf(stderr, "%.*s ", (int) ke->l, ke->p);
			}
			fprintf(stderr, "\n");
		} else {
			if (i + 2 < i1 && testReg(le[i].i)) {
				if (i + 4 < i1 && le[i + 1].i == phr->code[2] && testReg(le[i + 2].i) && le[i + 3].i == phr->code[2] && testConstInt(le[i + 4].i)) {
					// reg=reg+reg+imm
					j = getConstInt(le[i + 4].i);
					if (-128 <= j && j <= 127)
						c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i8", 1, (int) reg, 4, 0, (int) (le[i].i - env->sysReg0), (int) (le[i + 2].i - env->sysReg0), (int) j);
					else
						c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i32", 2, (int) reg, 4, 0, (int) (le[i].i - env->sysReg0), (int) (le[i + 2].i - env->sysReg0), j);
					i += 5 - 1;
					f = 1;
					continue;
				}
				if (i + 4 < i1 && le[i + 1].i == phr->code[2] && testReg(le[i + 2].i) && le[i + 3].i == phr->code[3] && testConstInt(le[i + 4].i)) {
					// reg=reg+reg-imm
					j = - getConstInt(le[i + 4].i);
					if (-128 <= j && j <= 127)
						c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i8", 1, (int) reg, 4, 0, (int) (le[i].i - env->sysReg0), (int) (le[i + 2].i - env->sysReg0), (int) j);
					else
						c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i32", 2, (int) reg, 4, 0, (int) (le[i].i - env->sysReg0), (int) (le[i + 2].i - env->sysReg0), j);
					i += 5 - 1;
					f = 1;
					continue;
				}
				if (le[i + 1].i == phr->code[2] && testReg(le[i + 2].i) && reg != le[i].i - env->sysReg0) {
					// reg=reg+reg
					c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i8", 1, (int) reg, 4, 0, (int) (le[i].i - env->sysReg0), (int) (le[i + 2].i - env->sysReg0), 0);
					i += 3 - 1;
					f = 1;
					continue;
				}
				if (le[i + 1].i == phr->code[2] && testConstInt(le[i + 2].i) && reg != le[i].i - env->sysReg0) {
					// reg=reg+imm
					j = getConstInt(le[i + 2].i);
					if (le[i].i - env->sysReg0 != 4 && le[i].i - env->sysReg0 != 12) {
						if (-128 <= j && j <= 127)
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%i8", 1, (int) reg, (int) (le[i].i - env->sysReg0), (int) j);
						else
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%i32", 2, (int) reg, (int) (le[i].i - env->sysReg0), j);
					} else {
						if (-128 <= j && j <= 127)
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i8", 1, (int) reg, 4, 0, 4, (int) (le[i].i - env->sysReg0), (int) j);
						else
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i32", 2, (int) reg, 4, 0, 4, (int) (le[i].i - env->sysReg0), j);
					}
					i += 3 - 1;
					f = 1;
					continue;
				}
				if (le[i + 1].i == phr->code[3] && testConstInt(le[i + 2].i) && reg != le[i].i - env->sysReg0) {
					// reg=reg-imm
					j = - getConstInt(le[i + 2].i);
					if (le[i].i - env->sysReg0 != 4 && le[i].i - env->sysReg0 != 12) {
						if (-128 <= j && j <= 127)
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%i8", 1, (int) reg, (int) (le[i].i - env->sysReg0), (int) j);
						else
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%i32", 2, (int) reg, (int) (le[i].i - env->sysReg0), j);
					} else {
						if (-128 <= j && j <= 127)
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i8", 1, (int) reg, 4, 0, 4, (int) (le[i].i - env->sysReg0), (int) j);
						else
							c += kputBin86(&q[c], "%rx1_8d_%mrm_%sib_%i32", 2, (int) reg, 4, 0, 4, (int) (le[i].i - env->sysReg0), j);
					}
					i += 3 - 1;
					f = 1;
					continue;
				}
			}

//			OpCall oc(&c, q, le, 0);
			OpCall oc; oc.pqc = &c; oc.code = q; oc.le = le; oc.pos = 0;
			if (le[i].i == phr->code[9]) {
				OpCall_put(&oc, "c", BRG(sub_f), 0);
				c += loadReg0(reg, env->sysReg0 + R00, &q[c]);
			} else if (le[i].i == phr->code[10]) {
				OpCall_put(&oc, "c", BRG(sub_f), 1);
				c += loadReg0(reg, env->sysReg0 + R00, &q[c]);
			} else if (le[i].i == phr->code[11]) {
				OpCall_put(&oc, "c", BRG(sub_f), 2);
				c += loadReg0(reg, env->sysReg0 + R00, &q[c]);
			} else if (i + 2 < i1 && le[i + 1].i == phr->code[17]) {
				KInt j1 = i + 1;
				int n = 0;
				do {
					j1++;
					if (le[j1].i == phr->code[17]) n++;
					if (le[j1].i == phr->code[18]) n--;
				} while (j1 < i1 && n >= 0);
				if (n >= 0) goto err;
				KInt pos1[2];
				pos1[0] = i + 2;
				pos1[1] = j1;
				// ̂ƂAz̃ANZX\͍Ȃ.
				c += loadReg(reg, &q[c], sid, le, pos1);
				c += loadReg0(R02, le[i].i, &q[c]);
				c += kputBin86(&q[c], "%rx1_c1_%mrm_%i8", 3, 4, (int) reg, kpopCount32(sizeof (KInt) - 1));
				if (env->lwait >= 0 && envl->ndbg == 0) {
					c += kputBin86(&q[c], "%rx1_3b_%mrm", 0, (int) reg, R02);
					c += execTimeError(&q[c], 0x72);
				}
				c += kputBin86(&q[c], "%rx1_03_%mrm_%i8", 1, (int) reg, R02, (int) (2 * sizeof (KInt)));
				c += kputBin86(&q[c], "%rx1_8b_%mrm_00", 1, (int) reg, (int) reg);
				i = j1;
			} else if (i + 3 < i1 && le[i + 1].i == phr->code[22] && le[i + 2].i == phr->code[17]) {
				KInt j1 = i + 2;
				int n = 0;
				do {
					j1++;
					if (le[j1].i == phr->code[17]) n++;
					if (le[j1].i == phr->code[18]) n--;
				} while (j1 < i1 && n >= 0);
				if (n >= 0) goto err;
				KInt pos1[2];
				pos1[0] = i + 3;
				pos1[1] = j1;
				// ̂ƂAz̃ANZX\͍Ȃ.
				c += loadReg(reg, &q[c], sid, le, pos1);
				c += loadReg0(R02, le[i].i, &q[c]);
				c += kputBin86(&q[c], "%rx1_c1_%mrm_%i8", 3, 4, (int) reg, kpopCount32(sizeof (char) - 1));
				if (env->lwait >= 0 && envl->ndbg == 0) {
					c += kputBin86(&q[c], "%rx1_3b_%mrm", 0, (int) reg, R02);
					c += execTimeError(&q[c], 0x72);
				}
				c += kputBin86(&q[c], "%rx1_03_%mrm_%i8", 1, (int) reg, R02, (int) (2 * sizeof (KInt)));
				c += kputBin86(&q[c], "%rx1_0f_be_%mrm_00", 1, (int) reg, (int) reg);
				i = j1;
			} else if (i + 1 < i1 && (le[i].i == phr->code[19] || le[i].i == phr->code[20] || le[i].i == phr->code[21])) {
				KInt pos1[2];
				pos1[0] = i + 1;
				pos1[1] = i1;
				oc.pos = pos1;
				if (le[i].i == phr->code[19])
					OpCall_put(&oc, "0", BRG(sub_ff16sin));
				if (le[i].i == phr->code[20])
					OpCall_put(&oc, "0", BRG(sub_ff16cos));
				if (le[i].i == phr->code[21])
					OpCall_put(&oc, "0", BRG(sub_ff16sqrt));
				c += loadReg0(reg, env->sysReg0 + R00, &q[c]);
				i = i1 - 1;
			} else if (i + 1 < i1 && le[i].i == phr->code[2] && testConstInt(le[i + 1].i)) {
				c += loadRegImm(reg, &q[c], getConstInt(le[i + 1].i));
				i++;
			} else if (i + 1 < i1 && le[i].i == phr->code[3] && testConstInt(le[i + 1].i)) {
				c += loadRegImm(reg, &q[c], - getConstInt(le[i + 1].i));
				i++;
			} else
				c += loadReg0(reg, le[i].i, &q[c]);
			f = 1;
		}
	}
	return c;
}

int OpCall_count = 0;

void OpCall_put(OpCall *w, const char *s,...)
{
	unsigned char *q = &w->code[*w->pqc];
	KInt qc = 0, i = 0, l, reg;
    va_list ap;
    va_start(ap, s);
	void *p = va_arg(ap, void *);
	if (OpCall_count != 0)
		printf("OpCall_put: warning: OpCall_count=%d\n", OpCall_count);
	OpCall_count++;
	while (*s != '\0') {
		char c = *s++;
		reg = R00;
		if (c == 'c') {
			l = va_arg(ap, KInt);
			qc += loadRegImm(reg, &q[qc], l);
		}
		if (c == 'P') {
			KInt v0 = 0, l = va_arg(ap, int) * 2;
			if (*s == '_') {
				v0 = va_arg(ap, KInt);
				s++;
			}
			if (w->pos[l] >= 0)
				qc += kputBin86(&q[qc], "%rx1_8d_%m0", (int) reg, R05, getVarOfs(w->le[w->pos[l]].i));
			else {
				if (v0 == (KInt) NOPARAM)
					break;
				qc += loadRegImm(reg, &q[qc], v0);
			}
		}
		if (c == 'E')
			qc += loadReg(reg, &q[qc], env->lx.sid, w->le, &w->pos[va_arg(ap, int) * 2]);
		if ('0' <= c && c <= '9') {
			KInt v0 = 0;
			if (*s == '_') {
				v0 = va_arg(ap, KInt);
				s++;
			}
			if (w->pos[(c - '0') * 2] >= 0) {
				l = (c - '0') * 2;
				if (testReg(w->le[w->pos[l]].i) && w->pos[l + 1] - w->pos[l] == 1)
					reg = w->le[w->pos[l]].i - env->sysReg0;
				else
					qc += loadReg(reg, &q[qc], env->lx.sid, w->le, &w->pos[l]);
			} else {
				if (v0 == (KInt) NOPARAM)
					break;
				qc += loadRegImm(reg, &q[qc], v0);
			}
		}
		if (c == 'S') {	// string(l, p)
		//	const char *t = w->le[w->pos[va_arg(ap, int) * 2]].p0;
			const char *t = KStringId_getStr(env->lx.sid, w->le[w->pos[va_arg(ap, int) * 2]].i)->p;
			for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
			if (env->mode_mkexe == 0) {
				qc += loadRegImm(reg, &q[qc], l);
				qc += kputBin86(&q[qc], "%rx1_89_%mrm_24_%i8", 1, (int) reg, R04, i * sizeof (KInt));
				i++;
				qc += loadRegImm(reg, &q[qc], (KInt) (t + 1));
			} else {
				int j = env->mkexe_txt.s;
				char *tt = printf_sub(l, t + 1);
				KVarArea_addBytes(&env->mkexe_txt, strlen(tt), tt);
				KMemPool_free(&kmemPool, l + 1, tt);
				KVarArea_addChar(&env->mkexe_txt, 0);
				qc += kputBin86(&q[qc], "%i8_%i32", reg + 0xb8, j + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, (&q[qc] - env->code - 4) << 4 | 8);
			}
		}
		if (i == 0)
			qc += kputBin86(&q[qc], "%rx1_89_%mrm_24", 0, (int) reg, 4);
		else
			qc += kputBin86(&q[qc], "%rx1_89_%mrm_24_%i8", 1, (int) reg, R04, i * sizeof (KInt));
		i++;
	}
    va_end(ap);
	OpCall_count--;
	#if (defined(KUSE64))
		#if (defined(KABI_MS64))
			if (i >= 1) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24",    0, R01, 4); // RCX
			if (i >= 2) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_08", 1, R02, 4); // RDX
			if (i >= 3) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_10", 1,   8, 4); // R08
			if (i >= 4) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_18", 1,   9, 4); // R09
		#elif (defined(KABI_SYSV64))
			qc += kputBin86(&q[qc], "%rx1_89_%mrm_24_%i32", 2, R07, 4, 240);	// save RDI
			if (i >=  1) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24",    0, R07, 4); // RDI
			if (i >=  2) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_08", 1, R06, 4); // RSI
			if (i >=  3) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_10", 1, R02, 4); // RDX
			if (i >=  4) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_18", 1, R01, 4); // RCX
			if (i >=  5) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_20", 1,   8, 4); // R08
			if (i >=  6) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_28", 1,   9, 4); // R09
			if (i >=  7) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_30  %rx1_89_%mrm_24",    1, R00, 4, 0, R00, 4);
			if (i >=  8) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_38  %rx1_89_%mrm_24_08", 1, R00, 4, 1, R00, 4);
			if (i >=  9) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_40  %rx1_89_%mrm_24_10", 1, R00, 4, 1, R00, 4);
			if (i >= 10) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_48  %rx1_89_%mrm_24_18", 1, R00, 4, 1, R00, 4);
			if (i >= 11) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_50  %rx1_89_%mrm_24_20", 1, R00, 4, 1, R00, 4);
			if (i >= 12) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_58  %rx1_89_%mrm_24_28", 1, R00, 4, 1, R00, 4);
			if (i >= 13) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_60  %rx1_89_%mrm_24_30", 1, R00, 4, 1, R00, 4);
			if (i >= 14) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_68  %rx1_89_%mrm_24_38", 1, R00, 4, 1, R00, 4);
			if (i >= 15) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_70  %rx1_89_%mrm_24_40", 1, R00, 4, 1, R00, 4);
			if (i >= 16) qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_78  %rx1_89_%mrm_24_48", 1, R00, 4, 1, R00, 4);
			if (i >= 17)
				kerrorExit("OpCall_put: internal error");
		#endif
	#endif
	if (env->mode_mkexe == 0)
		qc += kputBin86(&q[qc], "e8_%r32", p);
	else {
		if (p == BRG(sub_printf)) {
			qc += kputBin86(&q[qc], "ff_95_%i32", DLL_PRINTF);
			KVarArea_addInt(&env->mkexe_dll, &q[qc] - env->code);
		}
	}
	#if (defined(KUSE64) && defined(KABI_SYSV64))
		qc += kputBin86(&q[qc], "%rx1_8b_%mrm_24_%i32", 2, R07, 4, 240);	// load RDI
	#endif
	*w->pqc += qc;
}

KInt evalAdd(KInt l, const char *s);

KInt evalAdd1(const char *s)
{
	return evalAdd(strlen(s), s);
}

void eval0(KInt psel)
{
	EnvLocal *envl0 = envl;
	envl = KMemPool_allocC(&kmemPool, sizeof (EnvLocal));
	envl->envl0 = envl0;
	envl->progselm = psel;
	envl->qc0 = env->qc;
//	envl->err = 0;
//	envl->ndbg = 0;
	KVarArea_init(&envl->UsrStk, &kmemPool, 0);
	KVarArea_init(&envl->alias, &kmemPool, 0);
	KVarArea_initAd(&envl->stkFor, &kmemPool, 0);
	KVarArea_initAd(&envl->stkIf, &kmemPool, 0);
	KVarArea_initAd(&envl->gotoLink, &kmemPool, 0);
	KVarArea_initAd(&envl->cmplTime, &kmemPool, 0);
	KInt i, i1 = *env->lx.sid->n;
	for (i = 0; i < i1; i++)
		KVarArea_addInt(&envl->alias, i);
	KIndexDict0_init(&envl->label, &kmemPool, sizeof (KInt), 0, 0);
	envl->connect_c = 0xffffff;
	envl->connect_x = -0x7fffffff;
	if (env->mode_mkexe == 0) {
		#if (defined(KUSE64))
			env->qc += kputBin86(&env->code[env->qc], "41_57  41_56  41_55  41_54  41_53  41_52");
			env->qc += kputBin86(&env->code[env->qc], "41_51  41_50  57  56  55  54  53  52  51  50");
			env->qc += kputBin86(&env->code[env->qc], "%rx1_81_ec_f8_00_00_00"); // PUSHAD.  SUB ESP,248.  ̖߂ŃX^bN𒲐.
			env->qc += kputBin86(&env->code[env->qc], "%rx1_%i8_%i64", 0xb8 + 5, (KInt) 0);
		#elif (defined(KUSE32))
			env->qc += kputBin86(&env->code[env->qc], "60  83_ec_7c"); // PUSHAD.  SUB ESP,124.  ̖߂ŃX^bN𒲐.
			env->qc += kputBin86(&env->code[env->qc], "%i8_%i32", 0xb8 + 5, (KInt) 0);
		#endif
		env->qc += kputBin86(&env->code[env->qc], "e8_%r32", (KInt) BRG(sub_getPR04_0));
		env->qc += kputBin86(&env->code[env->qc], "%rx1_8b_%mrm", 0, R02, R00);
		env->qc += kputBin86(&env->code[env->qc], "%rx1_85_%mrm", 3, R02, R02);
		#if (defined(KUSE64))
			env->qc += kputBin86(&env->code[env->qc], "75_03");
		#elif (defined(KUSE32))
			env->qc += kputBin86(&env->code[env->qc], "75_02");
		#endif
		env->qc += kputBin86(&env->code[env->qc], "%rx1_89_%mrm", 3, R04, R02);
		env->qc += kputBin86(&env->code[env->qc], "%rx1_89_%mrm", 0, R02, R00);
		env->qc += kputBin86(&env->code[env->qc], "e8_%r32", (KInt) env->loadRegs);
		// [Jϐ̂Ƃ͏l. ͍lȂ.
	} else {
		env->qc += kputBin86(&env->code[env->qc], "55 89_e5 60 83_ec_78 bd_00_00_00_00");
	}
	envl->stkFor.s = 0;
	envl->stkIf.s = 0;
	envl->gotoLink.s = 0;
	#if (defined(KUSE64))
		evalAdd1("ALIAS *: : *,R00:RAX,R01:RCX,R02:RDX,R03:RBX,R04:RSP,R05:RBP,R06:RSI,R07:RDI");
	#elif (defined(KUSE32))
		evalAdd1("ALIAS *: : *::,R00:EAX,R01:ECX,R02:EDX,R03:EBX,R04:ESP,R05:EBP,R06:ESI,R07:EDI");
	#endif
	if (psel >= 0) {
		Prog *pp = prog_getProg(psel);
		if (pp != 0) {
			if (pp->lc == 0)
				pp->lc = KVarArea_init(0, &kmemPool, 0);
			KVarArea_reserve(pp->lc, KVarArea_sizPtr(&pp->src.ln.va) * sizeof (KInt) * 2);
			pp->lc->s = KVarArea_sizPtr(&pp->src.ln.va) * sizeof (KInt) * 2;
			memset(pp->lc->p, 0, pp->lc->s);	// resetLineExecCount.
		}
	}
}

KInt pF_regNnum, pF_rmNum, pF_bas, pF_idx, pF_scl, pF_dsp;

KInt phrFucSub0(KPhraseCmp *w, KLexer_Elmt *le, KInt pc, KInt i, KInt typ)
{
	KInt lpc = pc, j;
	if ((i & 1) == 0 && le[lpc].i != w->code[typ + 14]) return -1;
	if (le[lpc].i == w->code[typ + 14]) lpc++;
	if (le[lpc].i != w->code[2]) return -1;
	lpc++;
	pF_bas = pF_idx = -1;
	pF_scl = pF_dsp = 0;
	//
	for (;;) {
		if (le[lpc].i == w->code[6]) return -1;
		if (le[lpc].i == w->code[3]) break;
		j = KPhraseCmp_cmpGrp(w, 502, le[lpc].i) | 64;
		if (j < 0)
			j = KPhraseCmp_cmpGrp(w, 503, le[lpc].i) | 96;
		if (j < 0 && le[lpc].i == w->code[18])
			j = 16 | 96;
		if (j >= 0) {
			if (le[lpc + 1].i == w->code[19] && testConstInt(le[lpc + 2].i) != 0 && pF_scl == 0 && j != (16 | 96)) {
				pF_scl = getConstInt(le[lpc + 2].i);
				if (pF_idx != 1 && pF_idx != 2 && pF_idx != 4 && pF_idx != 8) return -1;
				pF_idx = j;
				lpc += 3;
				continue;
			}
			if (pF_bas >= 0 && pF_scl == 0 && j != (16 | 96)) {
				pF_scl = 1;
				pF_idx = j;
				lpc++;
				continue;
			}
			if (pF_bas < 0) {
				pF_bas = j;
				lpc++;
				continue;
			}
			return -1;
		}
		if (le[lpc].i == w->code[20]) {	// +
			lpc++;
			continue;
		}
		if (testConstInt(le[lpc].i) != 0) {
			pF_dsp += getConstInt(le[lpc].i);
			lpc++;
			continue;
		}
		if (le[lpc].i == w->code[21] && testConstInt(le[lpc + 1].i) != 0) {	// -
			pF_dsp -= getConstInt(le[lpc + 1].i);
			lpc += 2;
			continue;
		}
		if (pF_bas >= 0) return -1;
		#if (defined(KUSE64))
			pF_bas = 5 | 96;
			if (env->mode_mkexe == 0)
				pF_bas = 5 | 64;
		#elif (defined(KUSE32))
			pF_bas = 5 | 64;
		#endif
		pF_dsp += getVarOfs(le[lpc].i);
		lpc++;
	}
	pF_rmNum = typ << 5 | 256;
	return lpc + 1 - pc;
}

KInt phrFnc(KInt cmd, KInt i, KPhraseCmp *phr)
{
	if (phr->code[31] == 0)
		KPhraseCmp_cmp(phr, 0, "@ ( ) [ ] { } ; , == != < <= > >= BYTE WORD DWORD QWORD RIP * + -", 0, 0, 0, 0);
	if (cmd == 0) {
		if (i == phr->code[0]) return 0;
		if (i == phr->code[1]) return 0 + 16;
		if (i == phr->code[2]) return 1;
		if (i == phr->code[3]) return 1 + 16;
		if (i == phr->code[4]) return 2;
		if (i == phr->code[5]) return 2 + 16;
		return -1;
	}
	if (cmd == 1) {
		int j;
		for (j = 6; j < 14; j++) {
			if (i == phr->code[j])
				return -1;
		}
	}
	if (cmd == 2) {
		const char *s = (const char *) i;
		if (phr->fpc == 4 && strncmp(s, "@rUS", 4) == 0) return -256;
		if (phr->fpc == 4 && strncmp(s, "@imm", 4) == 0) return -259;
		if (phr->fpc == 3 && strncmp(s, "@cc", 3) == 0) return -260;

		if (phr->fpc == 3 && strncmp(s, "@r8", 3) == 0) return -950;
		if (phr->fpc == 4 && strncmp(s, "@r16", 4) == 0) return -951;
		if (phr->fpc == 4 && strncmp(s, "@r32", 4) == 0) return -952;
		if (phr->fpc == 4 && strncmp(s, "@r64", 4) == 0) return -953;

		if (phr->fpc == 4 && strncmp(s, "@rm8", 4) == 0) return -954;
		if (phr->fpc == 5 && strncmp(s, "@rm8_", 5) == 0) return -955;
		if (phr->fpc == 5 && strncmp(s, "@rm16", 5) == 0) return -956;
		if (phr->fpc == 6 && strncmp(s, "@rm16_", 6) == 0) return -957;
		if (phr->fpc == 5 && strncmp(s, "@rm32", 5) == 0) return -958;
		if (phr->fpc == 6 && strncmp(s, "@rm32_", 6) == 0) return -959;
		if (phr->fpc == 5 && strncmp(s, "@rm64", 5) == 0) return -960;
		if (phr->fpc == 6 && strncmp(s, "@rm64_", 6) == 0) return -961;

		kerrorExit("phrFnc: cmd=2: internal error");
	}
	if (cmd == 3) {	// 	F肵tokenԂ.
		int pc = phr->fpc;
		KLexer_Elmt *le = phr->fle;
		if (i == -256) { if (testReg(le[pc].i)) return 1; else return -1; }	// @rUS
		if (i == -259) { if (testConstInt(le[pc].i) != 0) return 1; else return -1; }		// @imm
		if (i == -260) { if (phr->code[8] <= le[pc].i && le[pc].i <= phr->code[8] + 5) return 1; else return -1; }		// @imm
		if (i == -950) { if ((pF_regNnum = KPhraseCmp_cmpGrp(phr, 500, le[pc].i) |  0) >= 0) return 1; else return -1; }		// @r8
		if (i == -951) { if ((pF_regNnum = KPhraseCmp_cmpGrp(phr, 501, le[pc].i) | 32) >= 0) return 1; else return -1; }		// @r16
		if (i == -952) { if ((pF_regNnum = KPhraseCmp_cmpGrp(phr, 502, le[pc].i) | 64) >= 0) return 1; else return -1; }		// @r32
		if (i == -953) { if ((pF_regNnum = KPhraseCmp_cmpGrp(phr, 503, le[pc].i) | 96) >= 0) return 1; else return -1; }		// @r64
		if (-955 <= i && i <= -954) {	// @rm8, @rm8_
			if ((pF_rmNum = KPhraseCmp_cmpGrp(phr, 500, le[pc].i) |  0) >= 0) return 1;
			return phrFucSub0(phr, le, pc, i, 0);
		}
		if (-957 <= i && i <= -956) {	// @rm16, @rm16_
			if ((pF_rmNum = KPhraseCmp_cmpGrp(phr, 501, le[pc].i) | 32) >= 0) return 1;
			return phrFucSub0(phr, le, pc, i, 1);
		}
		if (-959 <= i && i <= -958) {	// @rm32, @rm32_
			if ((pF_rmNum = KPhraseCmp_cmpGrp(phr, 502, le[pc].i) | 64) >= 0) return 1;
			return phrFucSub0(phr, le, pc, i, 2);
		}
		if (-961 <= i && i <= -960) {	// @rm64, @rm64_
			if ((pF_rmNum = KPhraseCmp_cmpGrp(phr, 503, le[pc].i) | 96) >= 0) return 1;
			return phrFucSub0(phr, le, pc, i, 3);
		}
		kerrorExit("phrFnc: cmd=3: internal error");
	}
	return 0;
}

const char *pBx(void *w, const char *s, unsigned char *q, KInt *pc, va_list *pap, unsigned char *rx)
{
	(void) w;
	(void) rx;
    va_list ap = *pap;
	KInt c = *pc;
	if (strncmp(s, "%rmr", 4) == 0) {
		if ((pF_regNnum & 8) != 0)
			*rx |= 0x04;
rmrc:
		if (pF_rmNum >= 0 && (pF_rmNum & 256) == 0) {
			q[c++] = 0xc0 | (pF_rmNum & 7) | (pF_regNnum & 7) << 3;
			if ((pF_rmNum & 8) != 0)
				*rx |= 0x01;
		} else {
			if (pF_bas >= 0 && pF_scl == 0 && (pF_bas & 7) != 4) {
				KInt bas = pF_bas & 7, mod = 2;
				if ((pF_bas & 8) != 0) *rx |= 1;
				if (-128 <= pF_dsp && pF_dsp <= 127) mod = 1;
				if (pF_dsp == 0) mod = 0;
				if ((pF_bas & 7) == 5 && mod == 0) mod = 1;
				if (pF_bas == (16 | 96)) {
					bas = 5;
					mod = 2;
					q[c++] = 0x00 | 5 | (pF_regNnum & 7) << 3;
				} else
					q[c++] = mod << 6 | bas | (pF_regNnum & 7) << 3;
				if (mod == 1)
					q[c++] = pF_dsp;
				if (mod == 2) {
					q[c]     =  pF_dsp        & 0xff;
					q[c + 1] = (pF_dsp >>  8) & 0xff;
					q[c + 2] = (pF_dsp >> 16) & 0xff;
					q[c + 3] = (pF_dsp >> 24) & 0xff;
					c += 4;
				}
			} else
				kerrorExit("sib");
		}
		s += 4;
		goto fin;
	}
	if (strncmp(s, "%rmc", 4) == 0) {
		pF_regNnum = va_arg(ap, KInt);
		goto rmrc;
	}
	return 0;
fin:
	*pap = ap;
	*pc = c;
	return s;
}

KInt cmpJmp(unsigned char *cod, KLexer_Elmt *le, KInt *ps, char flg, void *jmp, KStringId *sid, KInt cc0)
{
	static const unsigned char t[6] = { 0x84, 0x85, 0x8c, 0x8f, 0x8e, 0x8d }; // ==(JE), !=(JNE), <(JL), >(JG), <=(JLE), >=(JGE).
	KInt qc = 0, reg;
	if (ps[0] < ps[1]) {
		if (testReg(le[ps[0]].i) && ps[1] - ps[0] == 1)
			reg = le[ps[0]].i - env->sysReg0;
		else {
			reg = R00;
			qc += loadReg(R00, &cod[qc], sid, le, &ps[0]);
		}
		if (testConstInt(le[ps[4]].i)) {
			KInt i = getConstInt(le[ps[4]].i);
			if (i == 0)
				qc += kputBin86(&cod[qc], "%rx1_85_%mrm", 3, (int) reg, (int) reg);
			else if (-128 <= i && i <= 127)
				qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 7, (int) reg, i);
			else
				qc += kputBin86(&cod[qc], "%rx1_81_%mrm_%i32", 3, 7, (int) reg, i);
		} else if (testReg(le[ps[4]].i))
			qc += kputBin86(&cod[qc], "%rx1_39_%mrm", 3, le[ps[4]].i - env->sysReg0, (int) reg);
		else
			qc += kputBin86(&cod[qc], "%rx1_3b_%m0", (int) reg, R05, getVarOfs(le[ps[4]].i));
	}
	qc += kputBin86(&cod[qc], "0f_%i8_%r32", t[le[ps[2]].i - cc0] ^ flg, jmp);
	return qc;
}

void putKInt(unsigned char *p, KInt i);
int cmplTimeRun0(KVarArea *va);

KInt evalAdd(KInt l, const char *s)
{
	static KPhraseCmp *phr;
	static int cc0, semi;
	KLexer *lx = &env->lx;
	if (phr == 0) {
		phr = KPhraseCmp_init(0, &kmemPool, 32, 900, lx->sid, lx->chrTyp, phrFnc);
		cc0  = KStringId_getId0(lx->sid, "==");
		semi = KStringId_getId1(lx->sid, ";");
		KPhraseCmp_setGrp(phr, 500, "AL CL DL BL AH CH DH BH");
		KPhraseCmp_setGrp(phr, 501, "AX CX DX BX SP BP SI DI");
		KPhraseCmp_setGrp(phr, 502, "EAX ECX EDX EBX ESP EBP ESI EDI");
		KPhraseCmp_setGrp(phr, 503, "RAX RCX RDX RBX RSP RBP RSI RDI R08 R09 R10 R11 R12 R13 R14 R15");
		KPhraseCmp_setGrp(phr, 504, "ADD OR ADC SBB AND SUB XOR CMP");
		KPhraseCmp_setGrp(phr, 505, "RAX RCX RDX RBX RSP RBP RSI RDI");
		KPhraseCmp_setGrp(phr, 506, "R08 R09 R10 R11 R12 R13 R14 R15");
		KPhraseCmp_setGrp(phr, 507, "JO  JNO JC JNC JZ JNZ JBE JA   JS  JNS JP  JNP JL   JGE");
		KPhraseCmp_setGrp(phr, 508, "Jcc Jcc JB JNB JE JNE JNA JNBE Jcc Jcc Jcc Jcc JNGE JNL");
		KPhraseCmp_setGrp(phr, 509, "ROL ROR RCL RCR SHL SHR SHL SAR");
	}
	if (envl->err != 0) return 1;
	if (s[l - 1] == '\n')
		l--;
	KStringId *sid = KLexer_reset(lx)->sid;
	KLexer_Elmt *le = (KLexer_Elmt *) KLexer_add(KLexer_addCont(lx, l, (const char *) s), 1, ";")->p;
	unsigned char *cod = env->code;
	KInt qc = env->qc, pc = 0, pc1 = KLexer_getElmtLen(lx), ps[20], reg, i, lpc;
	for (;;) {
		KPhraseCmp_setPr(phr, le, ps, pc, pc1);
		if (KPhraseCmp_cmpPr(phr,   1, "@ ALIAS")) {
			for (pc++;;) {
				phr->prI0 = pc;
				if (PC(phr,   2, "@ ;")) break;
				if (PC(phr,   3, "@ @t:@t")) {		KVarArea_setIntIdx(&envl->alias, le[pc].i, KVarArea_getIntIdx(&envl->alias, le[pc + 2].i)); pc += 3;
				} else if (PC(phr,   4, "@ ,")) {	pc++;
				} else {							KVarArea_setIntIdx(&envl->alias, le[pc].i, le[pc].i); pc++; }
			}
			pc++;
			continue;
		} else
			break;
	}
	for (i = pc; i < pc1; i++)
		le[i].i = KVarArea_getIntIdx(&envl->alias, le[i].i);
constLoop:
	for (i = pc; i < pc1; i++) {
		if (i + 2 < pc1 && le[i].i == env->sysRetVal + 1 && le[i + 1].i == env->sysRetVal + 2) {
			KInt j = i + 2, nc = 0;
			while (j < pc1 && nc >= 0) {
				if (le[j].i == env->sysRetVal + 2) nc++;
				if (le[j].i == env->sysRetVal + 3) nc--;
				j++;
			}
			char tmp[128];
			int tmpLen;
			if (j == i + 4) {
				tmpLen = sprintf(tmp, "0x%x", (int) *getVarPtr(le[i + 2].i));
				memmove(&le[i + 1], &le[j], (pc1 - j) * sizeof (KLexer_Elmt));
				pc1 -= 3;
			} else {
				KVarArea bkup;
				KVarArea_init(&bkup, &kmemPool, pc1 * sizeof (KLexer_Elmt));
				memcpy(bkup.p, le, (i + 1) * sizeof (KLexer_Elmt));
				memcpy(bkup.p + (i + 1) * sizeof (KLexer_Elmt), &le[j], (pc1 - j) * sizeof (KLexer_Elmt));
				KVarArea tmpSrc;
				KVarArea_init(&tmpSrc, &kmemPool, 0);
				KVarArea_addInt(&tmpSrc, 1);
				KVarArea_addInt(&tmpSrc, 0);
				KVarArea_addBytes(&tmpSrc, 9, "$RETVAL= ");
				KInt k;
				j--;
				for (k = i + 2; k < j; k++) {
					KStringId_Elmt *se = KStringId_getStr(sid, le[k].i);
					KVarArea_addBytes(&tmpSrc, se->l, se->p);
					KVarArea_addBytes(&tmpSrc, 1, " ");
				}
				KVarArea_setIntIdx(&tmpSrc, 1, tmpSrc.s - sizeof (KInt) * 2);
				if (cmplTimeRun0(&tmpSrc) != 0) {
					pc = i;
					goto err;
				}
				KVarArea_dest(&tmpSrc);
				pc1 -= j - i;
				memcpy(le, bkup.p, pc1 * sizeof (KLexer_Elmt));
				KVarArea_dest(&bkup);
				tmpLen = sprintf(tmp, "0x%x", (int) *getVarPtr(env->sysRetVal));
			}
			le[i].i = KStringId_getId1L(env->lx.sid, tmpLen, tmp);
			goto constLoop;
		}
	}

	KVarArea xl; KVarArea_init(&xl, &kmemPool, 0);
	OpCall oc; oc.pqc = &qc; oc.code = cod; oc.le = le; oc.pos = ps;
   	for (; pc < pc1; pc++) {
		KPhraseCmp_setPr(phr, le, ps, pc, pc1);
		if (PC(phr, 200, "@ ;")) continue;
		if (PC(phr, 201, "@ @t=@rUS;")) {
			qc += storeReg(le[ps[2]].i - env->sysReg0, le[ps[0]].i, &cod[qc]);
		} else if (PC(phr, 253, "@ @t=RGBCOL(@ex,@ex,@ex);")) {
			OpCall_put(&oc, "123", BRG(sub_rgbcol));
			qc += storeReg(R00, le[ps[0]].i, &cod[qc]);
		} else if (PC(phr, 254, "@ @t=RGBCOL @ex,@ex,@ex;")) {
			OpCall_put(&oc, "123", BRG(sub_rgbcol));
			qc += storeReg(R00, le[ps[0]].i, &cod[qc]);
		} else if (PC(phr, 203, "@ @t=@ex;")) {
			reg = selectReg(le[pc].i, R00);
			qc += loadReg(reg, &cod[qc], sid, le, &ps[2]);
			qc += storeReg(reg, le[pc].i, &cod[qc]);
		} else if (PC(phr, 204, "@ EXIT;")) {		OpCall_put(&oc, "", BRG(sub_exit));
		} else if (PC(phr, 205, "@ LIST;")) {		OpCall_put(&oc, "", BRG(sub_list));
		} else if (PC(phr, 206, "@ PRINT")) {
			for (lpc = pc +1;;) {
				phr->prI0 = lpc;
				if (PC(phr, 207, "@ ;")) {			OpCall_put(&oc, "", BRG(sub_printLf)); break; }
				if (PC(phr, 208, "@ ,;")) {			OpCall_put(&oc, "", BRG(sub_printSpc));  break; }
				if (PC(phr, 209, "@ ,")) {			OpCall_put(&oc, "", BRG(sub_printSpc)); lpc++; continue; }
				if (PC(phr, 210, "@ @cs")) {		OpCall_put(&oc, "S", BRG(sub_printStr), 0); lpc++; }
				else { PC(phr, 211, "@ @ex");		OpCall_put(&oc, "0", BRG(sub_print)); lpc += ps[1] - ps[0]; }
				phr->prI0 = lpc;
				if (PC(phr, 212, "@ ,;")) break;
				if (PC(phr, 213, "@ ,")) lpc++;
			}
		} else if (PC(phr, 214, "@ FOR @t=@ex,@t @_; ,@t_;")) {
			reg = selectReg(le[ps[0]].i, R00);
			qc += loadReg(reg, &cod[qc], sid, le, &ps[2]);
			qc += storeReg(reg, le[ps[0]].i, &cod[qc]);
			StkFor *st = KVarArea_reserve1(&envl->stkFor, sizeof (StkFor));
//while (((KInt) &cod[qc]) % 16 > 0) cod[qc++] = 0x90;
			st->qc = qc;
			st->c = le[ps[0]].i;
			st->c1 = le[ps[4]].i;
			st->d = -1;
			st->ne = 0;
			if (ps[6] >= 0)
				st->d = le[ps[6]].i;
		} else if (PC(phr, 215, "@ FORNE @t=@ex,@t @_; ,@t_;")) {
			reg = selectReg(le[ps[0]].i, 0);
			qc += loadReg(reg, &cod[qc], sid, le, &ps[2]);
			qc += storeReg(reg, le[ps[0]].i, &cod[qc]);
			StkFor *st = KVarArea_reserve1(&envl->stkFor, sizeof (StkFor));
//while (((KInt) &cod[qc]) % 16 > 0) cod[qc++] = 0x90;
			st->qc = qc;
			st->c = le[ps[0]].i;
			st->c1 = le[ps[4]].i;
			st->d = -1;
			st->ne = 1;
			if (ps[6] >= 0)
				st->d = le[ps[6]].i;
		} else if (PC(phr, 216, "@ NEXT;")) {
			if (envl->stkFor.s < (KInt) sizeof (StkFor))
				goto err;
			StkFor *st = (StkFor *) &envl->stkFor.p[envl->stkFor.s - sizeof (StkFor)];
			reg = selectReg(st->c, R00);
			if (!testReg(st->c))
				qc += kputBin86(&cod[qc], "%rx1_8b_%mrm_%i32", 2, (int) reg, R05, getVarOfs(st->c));
			if (st->d < 0)
				qc += kputBin86(&cod[qc], "%rx1_83_%mrm_01", 3, 0, (int) reg);
			else if (testConstInt(st->d)) {
				KInt i = getConstInt(st->d);
				if (-128 <= i && i <= 127 && i != 0)
					qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 0, (int) reg, (int) i);
				else if (i != 0)
					qc += kputBin86(&cod[qc], "%rx1_81_%mrm_%i32", 3, 0, (int) reg, i);
			} else
				qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i32", 2, (int) reg, R05, getVarOfs(st->d));
			qc += storeReg(reg, st->c, &cod[qc]);
			if (testConstInt(st->c1))
				qc += kputBin86(&cod[qc], "%rx1_81_%mrm_%i32", 3, 7, (int) reg, getConstInt(st->c1));
			else {
				int reg1 = R01;
				if (!testReg(st->c1))
					qc += kputBin86(&cod[qc], "%rx1_8b_%mrm_%i32", 2, (int) reg1, R05, getVarOfs(st->c1));
				else
					reg1 = st->c1 - env->sysReg0;
				qc += kputBin86(&cod[qc], "%rx1_39_%mrm", 3, (int) reg1, (int) reg);
			}
			if (st->d >= 0 && testConstInt(st->d) && getConstInt(st->d) < 0) {
				if (st->ne == 0)
					qc += kputBin86(&cod[qc], "0f_8d_%r32", &cod[st->qc]); // >=(JGE).
				else
					qc += kputBin86(&cod[qc], "0f_8f_%r32", &cod[st->qc]); // >(JG).
			} else {
				if (st->ne == 0)
					qc += kputBin86(&cod[qc], "0f_8e_%r32", &cod[st->qc]); // <=(JLE).
				else
					qc += kputBin86(&cod[qc], "0f_8c_%r32", &cod[st->qc]); // <(JL).
			}
			envl->stkFor.s -= sizeof (StkFor);
		} else if (PC(phr, 217, "@ LABEL @t;") != 0 || PC(phr, 318, "@ @t:;") != 0) {
			if (KIndexDict0_search1(&envl->label, &le[ps[0]].i) == 0) {
				KInt *label = (KInt *) KMemPool_alloc(&kmemPool, 2 * sizeof (KInt));
				label[0] = ~le[ps[0]].i;
				label[1] = qc;
				KIndexDict0_ins(&envl->label, label);
			} else {
				KStringId_Elmt *se = KStringId_getStr(sid, le[ps[0]].i);
				fprintf(stderr, "label warning : %.*s\n", (int) se->l, se->p);
			}
		} else if (PC(phr, 218, "@ GOTO @t;") != 0 && envl->progselm >= 0) {
			qc += kputBin86(&cod[qc], "e9_%r32", 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[0]].i));
		} else if (PC(phr, 219, "@ IF @ex @cc @t GOTO @t;") != 0 && envl->progselm >= 0) {
			qc += cmpJmp(&cod[qc], le, &ps[0], 0, 0, sid, cc0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[6]].i));
		} else if (PC(phr, 220, "@ IF @ex @cc @t THEN")) {
			qc += cmpJmp(&cod[qc], le, &ps[0], 1, 0, sid, cc0);
			StkIf *st = KVarArea_reserve1(&envl->stkIf, sizeof (StkIf));
			st->qc0 = qc - 4;
			st->qc1 = -1;
			pc = ps[5];
			continue;
		} else if (PC(phr, 221, "@ ELSE")) {
			if (envl->stkIf.s < (KInt) sizeof (StkIf))
				goto err;
			StkIf *st = (StkIf *) &envl->stkIf.p[envl->stkIf.s - sizeof (StkIf)];
			qc += kputBin86(&cod[qc], "e9_%r32", 0);
			st->qc1 = qc - 4;
			kputBin86(&cod[st->qc0], "%r32", &cod[qc]);
			continue;
		} else if (PC(phr, 222, "@ FI;")) {
			if (envl->stkIf.s < (KInt) sizeof (StkIf))
				goto err;
			StkIf *st = (StkIf *) &envl->stkIf.p[envl->stkIf.s - sizeof (StkIf)];
			if (st->qc1 < 0)
				kputBin86(&cod[st->qc0], "%r32", &cod[qc]);
			else
				kputBin86(&cod[st->qc1], "%r32", &cod[qc]);
			envl->stkIf.s -= sizeof (StkIf);
		} else if (PC(phr, 223, "@ TIMEMODE @ex;")) {				OpCall_put(&oc, "0", BRG(sub_timemode));
		} else if (PC(phr, 224, "@ LWAIT @ex;")) {					OpCall_put(&oc, "0", BRG(sub_lwait));
		} else if (PC(phr, 225, "@ CODEDUMP @ex;")) {				OpCall_put(&oc, "0", BRG(sub_codedump));
		} else if (PC(phr, 226, "@ ALIGN @imm;")) {
			i = getConstInt(le[pc + 1].i);
			while (((KInt) &cod[qc]) % i > 0)
				cod[qc++] = 0x90;
		} else if (PC(phr, 227, "@ DOLOOP;")) {
			StkFor *st = KVarArea_reserve1(&envl->stkFor, sizeof (StkFor));
			st->qc = qc;
			st->ne = 2;
		} else if (PC(phr, 228, "@ ENDDO @ex @cc @t;")) {
			if (envl->stkFor.s < (KInt) sizeof (StkFor))
				goto err;
			StkFor *st = (StkFor *) &envl->stkFor.p[envl->stkFor.s - sizeof (StkFor)];
			qc += cmpJmp(&cod[qc], le, &ps[0], 0, &cod[st->qc], sid, cc0);
			envl->stkFor.s -= sizeof (StkFor);
		} else if (PC(phr, 229, "@ SETPIX @ex,@ex,@ex;")) {			OpCall_put(&oc, "012", BRG(sub_setpix));
		} else if (PC(phr, 230, "@ INPUT @t;")) {					OpCall_put(&oc, "P", BRG(sub_input), 0);
		} else if (PC(phr, 231, "@ INPUT @cs,@t;")) { 				OpCall_put(&oc, "S", BRG(sub_printStr), 0); OpCall_put(&oc, "P", BRG(sub_input), 1);
		} else if (PC(phr, 232, "@ RAND @t;")) {					OpCall_put(&oc, "P", BRG(sub_rand), 0);
		} else if (PC(phr, 233, "@ GCLS @_; @ex_;")) {				OpCall_put(&oc, "0_", BRG(sub_gcls), (KInt) 0);
		} else if (PC(phr, 234, "@ OPENWIN @ex,@ex;")) {			OpCall_put(&oc, "01", BRG(sub_openwin));
		} else if (PC(phr, 235, "@ FLUSHWIN;")) {					OpCall_put(&oc, "", BRG(sub_flushwin));
		} else if (PC(phr, 236, "@ LEAPFLUSHWIN @ex;")) {			OpCall_put(&oc, "0", BRG(sub_leapflushwin));
		} else if (PC(phr, 237, "@ EWAIT @ex;")) {					OpCall_put(&oc, "0", BRG(sub_ewait));
		} else if (PC(phr, 238, "@ EINKEY @t;")) {					OpCall_put(&oc, "P", BRG(sub_einkey), 0);
		} else if (PC(phr, 239, "@ GETCHR @ex,@ex,@ex;")) {			OpCall_put(&oc, "01P", BRG(sub_getchr), 2);
		} else if (PC(phr, 240, "@ GETCOL @ex,@ex,@ex;")) {			OpCall_put(&oc, "01P", BRG(sub_getcol), 2);
		} else if (PC(phr, 241, "@ CHRBOX @ex,@ex,@ex,@ex,@ex @_; ,@ex_,@ex_;")) {	OpCall_put(&oc, "012345_6_", BRG(sub_chrbox), (KInt) 7, (KInt) 0);
		} else if (PC(phr, 242, "@ CHROFS @ex,@ex;")) {				OpCall_put(&oc, "01", BRG(sub_chrofs));
		} else if (PC(phr, 243, "@ GPRINTS @ex,@ex,@cs @_; ,@ex_,@ex_;")) {		OpCall_put(&oc, "01S3_4_", BRG(sub_gprints), 2, (KInt) 7, (KInt) 0);
		} else if (PC(phr, 244, "@ GPRINTI @ex,@ex,@ex,@ex @_; ,@ex_,@ex_;")) {	OpCall_put(&oc, "01234_5_", BRG(sub_gprinti), (KInt) 7, (KInt) 0);
		} else if (PC(phr, 245, "@ SCROLL @ex;")) {					OpCall_put(&oc, "0", BRG(sub_scroll));
		} else if (PC(phr, 246, "@ FILLRECT0 @ex,@ex,@ex,@ex,@ex;")) {	OpCall_put(&oc, "01234", BRG(sub_fillrect));
		} else if (PC(phr, 247, "@ NEW @_; @ex_;")) {				OpCall_put(&oc, "0_", BRG(sub_new), (KInt) env->progseln);
		} else if (PC(phr, 248, "@ INITSTACK @ex;")) {				OpCall_put(&oc, "0", BRG(sub_initstack)); qc += kputBin86(&cod[qc], "%rx1_97");
		} else if (PC(phr, 249, "@ PUSH @ex;")) {
			reg = selectReg(le[ps[0]].i, R00);
			qc += loadReg(reg, &cod[qc], sid, le, &ps[0]);
			qc += kputBin86(&cod[qc], "%rx1_89_%mrm  %rx1_83_%mrm_%i8", 0, (int) reg, R07, 3, 0, R07, (int) sizeof (KInt));
		} else if (PC(phr, 250, "@ POP @t;")) {
			reg = selectReg(le[ps[0]].i, R00);
			qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8   %rx1_8b_%mrm", 3, 0, R07, (int) (0x100 - sizeof (KInt)), 0, (int) reg, R07);
			qc += storeReg(reg, le[ps[0]].i, &cod[qc]);
		} else if (PC(phr, 251, "@ GOSUB @t;") != 0 && envl->progselm >= 0) {
			#if (defined(KUSE64))
				qc += kputBin86(&cod[qc], "%rx1_b8_%i64", (KInt) &cod[qc + 22]);
			#elif (defined(KUSE32))
				qc += kputBin86(&cod[qc], "b8_%i32", (KInt) &cod[qc + 15]);
			#endif
			qc += kputBin86(&cod[qc], "%rx1_89_%mrm  %rx1_83_%mrm_%i8", 0, R00, R07, 3, 0, R07, (int) sizeof (KInt));
			qc += kputBin86(&cod[qc], "e9_%r32", 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[0]].i));
			qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 0, R07, (int) (0x100 - sizeof (KInt)));
		} else if (PC(phr, 252, "@ RETURN;") != 0 && envl->progselm >= 0) {
			qc += kputBin86(&cod[qc], "%rx1_ff_%mrm_%i8", 1, 4, R07, (int) (0x100 - sizeof (KInt)));
		} else if (PC(phr, 255, "@ ECHR @ex,@ex,@ex @_; ,@ex_,@ex_;")) {		OpCall_put(&oc, "0123_4_", BRG(sub_echr), (KInt) 7, (KInt) 0);
		} else if (PC(phr, 256, "@ @t[@ex]=@ex;")) {
			qc += loadReg(R02, &cod[qc], sid, le, &ps[2]);
			qc += loadReg0(R00, le[pc].i, &cod[qc]);
			qc += kputBin86(&cod[qc], "%rx1_c1_%mrm_%i8", 3, 4, R02, kpopCount32(sizeof (KInt) - 1));
			if (env->lwait >= 0 && envl->ndbg == 0) {
				qc += kputBin86(&cod[qc], "%rx1_3b_%mrm", 0, R02, R00);
				qc += execTimeError(&cod[qc], 0x72);
			}
			qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R02, R00, (int) (2 * sizeof (KInt)));
			qc += loadReg(R00, &cod[qc], sid, le, &ps[4]);
			qc += kputBin86(&cod[qc], "%rx1_89_%mrm_00", 1, R00, R02);
		} else if (PC(phr, 257, "@ ARY INT NEW @t[@ex];")) {		OpCall_put(&oc, "P1c", BRG(sub_aryintnew), 0, (KInt) sizeof (KInt));
		} else if (PC(phr, 258, "@ ARY INT DEL @t;")) {				OpCall_put(&oc, "0", BRG(sub_aryintdel));
		} else if (PC(phr, 259, "@ ARY INT @t[@ex...]={")) {		// ނ𓾂R01j󂷂.
			qc += loadReg(R02, &cod[qc], sid, le, &ps[2]);
			qc += loadReg(R00, &cod[qc], sid, le, &ps[0]);
			qc += kputBin86(&cod[qc], "%rx1_c1_%mrm_%i8", 3, 4, R02, kpopCount32(sizeof (KInt) - 1));
			if (env->lwait >= 0 && envl->ndbg == 0) {
				qc += kputBin86(&cod[qc], "%rx1_8b_%mrm_%i8", 1, R01, R00, (int) sizeof (KInt));
				qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R01, R00, (int) (2 * sizeof (KInt)));
			}
			qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R02, R00, (int) (2 * sizeof (KInt)));
			for (lpc = ps[3] + 3;;) {
				phr->prI0 = lpc;
				if (PC(phr, 260, "@ ;")) break;
				if (PC(phr, 261, "@ {")) { lpc++; continue; }
				if (PC(phr, 262, "@ } ;")) break;
				if (PC(phr, 263, "@ ,")) { lpc++; continue; }
				if (PC(phr, 264, "@ @ex") == 0) goto err;
				qc += loadReg(R00, &cod[qc], sid, le, &ps[0]);
				if (env->lwait >= 0 && envl->ndbg == 0) {
					qc += kputBin86(&cod[qc], "%rx1_3b_%mrm", 3, R02, R01);
					qc += execTimeError(&cod[qc], 0x72);
				}
				qc += kputBin86(&cod[qc], "%rx1_89_%mrm_00", 1, R00, R02); // MOV [RDX],RAX.
				qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 0, R02, (int) sizeof (KInt));	// ADD RDX,sizeof (KInt).
				lpc = ps[1];
			}
		} else if (PC(phr, 265, "@ END;")) {
			qc += kputBin86(&cod[qc], "e9_%r32", (KInt) env->exitForce);
		} else if (PC(phr, 266, "@ ENDDO;")) {
			if (envl->stkFor.s < (KInt) sizeof (StkFor))
				goto err;
			StkFor *st = (StkFor *) &envl->stkFor.p[envl->stkFor.s - sizeof (StkFor)];
			qc += kputBin86(&cod[qc], "e9_%r32", &cod[st->qc]);
			envl->stkFor.s -= sizeof (StkFor);
		} else if (PC(phr, 267, "@ PROGLIST;")) {					OpCall_put(&oc, "", BRG(sub_proglist));
		} else if (PC(phr, 268, "@ PROGSEL @ex;")) {				OpCall_put(&oc, "0", BRG(sub_progsel));
		} else if (PC(phr, 269, "@ PROGSELN @t;")) {				OpCall_put(&oc, "P", BRG(sub_progseln), 0);
		} else if (PC(phr, 270, "@ PROGDEL @ex;")) {				OpCall_put(&oc, "0", BRG(sub_progdel));
		} else if (PC(phr, 271, "@ PROGNEW @t;")) {					OpCall_put(&oc, "P", BRG(sub_prognew), 0);
		} else if (PC(phr, 272, "@ PROGRUN @ex_;")) {				OpCall_put(&oc, "0_", BRG(sub_run), (KInt) env->progseln);
		} else if (PC(phr, 273, "@ LWAIT @ex;")) {					OpCall_put(&oc, "0", BRG(sub_lwait));
		} else if (PC(phr, 274, "@ LINECOUNTER @_; @ex_;")) {		OpCall_put(&oc, "0_", BRG(sub_linecounter), (KInt) env->progseln);
		} else if (PC(phr, 275, "@ RENUM @_; @ex_;")) {				OpCall_put(&oc, "c0_", BRG(sub_renum), (KInt) env->progseln, (KInt) 1000);
		} else if (PC(phr, 276, "@ STOP;")) {						qc += kputBin86(&cod[qc], "e8_%r32", (KInt) env->saveRegs); qc += stopSub(&cod[qc]);
		} else if (PC(phr, 277, "@ PAUSE;")) {
			qc += kputBin86(&cod[qc], "e8_%r32   b8_%i32   %rx1_89_%mrm_%i32   e8_%r32", env->saveRegs, (KInt) env->ln, 2, R00, R05, (KInt) getVarOfs(env->sysLine), BRG(sub_pause));
			qc += kputBin86(&cod[qc], "e8_%r32", env->loadRegs);
		} else if (PC(phr, 278, "@ NODEBUG @t;")) {
			if (testConstInt(le[ps[0]].i) == 0) goto err;
			KInt i = getConstInt(le[ps[0]].i);
			if (i < 0 || 1 < i) goto err;
			if (i == 0) envl->ndbg--; else envl->ndbg++;
			if (envl->ndbg < 0) goto err;
		} else if (PC(phr, 279, "@ RESETLC @_; @ex_;")) {			KInt i = envl->progselm; if (i < 0) i = env->progseln; OpCall_put(&oc, "0_", BRG(sub_resetlc), (KInt) i);
		} else if (PC(phr, 280, "@ NORETURN;")) {					qc += kputBin86(&cod[qc], "%rx1_83_%mrm_fc", 3, 0, R07);
		} else if (PC(phr, 281, "@ NORETURN @t;")) {
			qc += kputBin86(&cod[qc], "%rx1_83_%mrm_fc", 3, 0, R07);
			qc += kputBin86(&cod[qc], "e9_%r32", 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[0]].i));
		} else if (PC(phr, 282, "@ DEBUGTRAP @t;")) {				OpCall_put(&oc, "cc", BRG(sub_dtrp), (KInt) env->progseln, (KInt) le[ps[0]].i);
		} else if (PC(phr, 283, "@ ASSERT")) {
			if (env->lwait >= 0 && envl->ndbg == 0) continue;	// 1ꂾǂݔ΂(=ASSERTȍ~s).
			break;	// Z~R܂œǂݔ΂(=ASSERTȍ~sȂ).
		} else if (PC(phr, 284, "@ FILLRECT @ex,@ex,@ex,@ex,@ex;")) {		OpCall_put(&oc, "01234", BRG(sub_fillrectex));
		} else if (PC(phr, 285, "@ DRAWRECT @ex,@ex,@ex,@ex,@ex;")) {		OpCall_put(&oc, "01234", BRG(sub_drawrectex));
		} else if (PC(phr, 286, "@ DRAWLINE @ex,@ex,@ex,@ex,@ex;")) {		OpCall_put(&oc, "01234", BRG(sub_drawlineex));
		} else if (PC(phr, 287, "@ FILLOVAL @ex,@ex,@ex,@ex,@ex;")) {		OpCall_put(&oc, "01234", BRG(sub_fillovalex));
		} else if (PC(phr, 288, "@ DRAWOVAL @ex,@ex,@ex,@ex,@ex;")) {		OpCall_put(&oc, "01234", BRG(sub_drawovalex));
		} else if (PC(phr, 289, "@ FILL @ex,@ex,@ex;")) {					OpCall_put(&oc, "012", BRG(sub_fillex));
		} else if (PC(phr, 290, "@ DRAWLINE0 @ex,@ex,@ex,@ex,@ex;")) {		OpCall_put(&oc, "01234", BRG(sub_drawline));
		} else if (PC(phr, 291, "@ CLRKEYBUF;")) {							OpCall_put(&oc, "", BRG(sub_clrkeybuf));
		} else if (PC(phr, 292, "@ KWAIT @ex;")) {							OpCall_put(&oc, "0", BRG(sub_kwait));
		} else if (PC(phr, 293, "@ GETPIX @ex,@ex,@t;")) {					OpCall_put(&oc, "01P", BRG(sub_getpix), 2);
		} else if (PC(phr, 294, "@ SETGMODE @ex;")) {						OpCall_put(&oc, "0", BRG(sub_setgmode));
		} else if (PC(phr, 295, "@ BITBLT @ex,@ex,@ex,@ex,@ex @_; ,@ex_;")) { OpCall_put(&oc, "012345_", BRG(sub_bitblt), (KInt) 0);
		} else if (PC(phr, 296, "@ SAVEWIN @cs @_; ,@ex_,@ex_,@ex_,@ex_,@ex_;")) { OpCall_put(&oc, "S1_2_3_4_5_", BRG(sub_savewin), 0, (KInt) 0, (KInt) 0, (KInt) 0, (KInt) 0, (KInt) 0);
		} else if (PC(phr, 297, "@ CONNECT @_; @ex_;")) {					OpCall_put(&oc, "cc0_", BRG(sub_connect), (KInt) -0x7fffffff, (KInt) 0, (KInt) -1);
		} else if (PC(phr, 298, "@ CONNECT @ex,@ex @_; ,@ex_;")) {			OpCall_put(&oc, "012_", BRG(sub_connect), (KInt) -1);
		} else if (PC(phr, 299, "@ FILLOVALR @ex,@ex,@ex,@ex @_; ,@ex;")) {	OpCall_put(&oc, "01234_", BRG(sub_fillovalrex), (KInt) -1);
		} else if (PC(phr, 300, "@ DRAWOVALR @ex,@ex,@ex,@ex @_; ,@ex;")) {	OpCall_put(&oc, "01234_", BRG(sub_drawovalrex), (KInt) -1);
		} else if (PC(phr, 301, "@ BLTBIT @ex,@ex,@ex,@ex,@ex @_; ,@ex_;")) { OpCall_put(&oc, "012345_", BRG(sub_bltbit), (KInt) 0);
		} else if (PC(phr, 302, "@ BITBLTX @ex,@ex,@ex,@ex,@ex @_; ,@ex_,@ex_,@ex_;")) { OpCall_put(&oc, "012345_6_7_", BRG(sub_bitbltx), (KInt) 0, (KInt) 1, (KInt) 0);
		} else if (PC(phr, 303, "@ PRINTF @cs  @_; ,@ex_,@ex_,@ex_,@ex_,@ex_,@ex_;")) {
			OpCall_put(&oc, "S1_2_3_4_5_6_", BRG(sub_printf), 0, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM);
		} else if (PC(phr, 304, "@ PRINTFF16 @cs  @_; ,@ex_,@ex_,@ex_,@ex_,@ex_,@ex_;")) {
			OpCall_put(&oc, "S1_2_3_4_5_6_", BRG(sub_printff16), 0, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM, (KInt) NOPARAM);
		} else if (PC(phr, 305, "@ ESCTERM @ex @_; ,@ex_,@ex_,@ex_,@ex_;")) { OpCall_put(&oc, "01_2_3_4_", BRG(sub_escterm), (KInt) 0, (KInt) 0, (KInt) 0, (KInt) 0);
		} else if (PC(phr, 306, "@ ARY INT @t[@ex...]=@cs;")) {		// ނ𓾂R01j󂷂.
phr306:
			qc += loadReg(R02, &cod[qc], sid, le, &ps[2]);
			qc += loadReg(R00, &cod[qc], sid, le, &ps[0]);
			qc += kputBin86(&cod[qc], "%rx1_c1_%mrm_%i8", 3, 4, R02, kpopCount32(sizeof (KInt) - 1));
			if (env->lwait >= 0 && envl->ndbg == 0) {
				qc += kputBin86(&cod[qc], "%rx1_8b_%mrm_%i8", 1, R01, R00, (int) sizeof (KInt));
				qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R01, R00, (int) (2 * sizeof (KInt)));
			}
			qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R02, R00, (int) (2 * sizeof (KInt)));
			const char *t = KStringId_getStr(sid, le[ps[4]].i)->p;
			int l, m; for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
			t++;
			for (m = 0; m <= l; m++) {
				if (m < l)
					qc += loadRegImm(R00, &cod[qc], t[m]);
				else
					qc += loadRegImm(R00, &cod[qc], 0);
				if (env->lwait >= 0 && envl->ndbg == 0) {
					qc += kputBin86(&cod[qc], "%rx1_3b_%mrm", 3, R02, R01);
					qc += execTimeError(&cod[qc], 0x72);
				}
				qc += kputBin86(&cod[qc], "%rx1_89_%mrm_00", 1, R00, R02); // MOV [RDX],RAX.
				qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 0, R02, (int) sizeof (KInt));	// ADD RDX,sizeof (KInt).
			}
		} else if (PC(phr, 307, "@ RUN;")) {								OpCall_put(&oc, "c", BRG(sub_run), (KInt) env->progseln);
		} else if (PC(phr, 308, "@ LOADWIN @cs @_; ,@ex_,@ex_,@ex_;")) { OpCall_put(&oc, "S1_2_3_", BRG(sub_loadwin), 0, (KInt) 0, (KInt) 0, (KInt) 0);
		} else if (PC(phr, 309, "@ ARY INT DUMP6X @t;")) {					OpCall_put(&oc, "0c", BRG(sub_aryintdump6x), (KInt) le[ps[0]].i);
		} else if (PC(phr, 310, "@ ARY STATIC INT NEW @t[@imm];")) {
#if 0
			if (env->mode_mkexe == 0)
				OpCall_put(&oc, "P1c", BRG(sub_aryintnew), 0, (KInt) sizeof (KInt));
			else {
				KInt i = getVarOfs(le[ps[0]].i) + (0x80 + 0x24);
				KInt j = env->mkexe_idt.s;
				KVarArea_reserve1(&env->mkexe_idt, 3 * sizeof (KInt));
				KInt k = getConstInt(le[ps[2]].i);
				KInt l = env->mkexe_idt.s;
				KVarArea_reserve1C1(&env->mkexe_idt, k * sizeof (KInt));
				put32((unsigned char *) &env->mkexe_idt.p[i], j + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, i << 4 | 5);
				put32((unsigned char *) &env->mkexe_idt.p[j + 0 * sizeof (KInt)], k * sizeof (KInt));
				put32((unsigned char *) &env->mkexe_idt.p[j + 1 * sizeof (KInt)], k * sizeof (KInt));
				put32((unsigned char *) &env->mkexe_idt.p[j + 2 * sizeof (KInt)], l + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, (j + 2 * sizeof (KInt)) << 4 | 5);
			}
#endif
		} else if (PC(phr, 311, "@ ARY STATIC INT @t[@imm...]=@cs;")) {
#if 0
			if (env->mode_mkexe == 0) goto phr306;
			KInt i = getVarOfs(le[ps[0]].i) + (0x80 + 0x24), j, k;
			j = *(int*)&env->mkexe_idt.p[i] - 0x00401000;
			KInt n = *(int*)&env->mkexe_idt.p[j + 2 * sizeof (KInt)] - 0x00401000;
			const unsigned char *t = (const unsigned char *) KStringId_getStr(sid, le[ps[4]].i)->p;
			KInt l, m = getConstInt(le[ps[2]].i);
			for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
			t++;
			for (k = 0; k < l; k++)
				*(KInt*)&env->mkexe_idt.p[n + (m + k) * sizeof (KInt)] = t[k];
#endif
		} else if (PC(phr, 312, "@ GETARG @t @_; ,@t_,@t_,@t_;")) {
			if (env->mode_mkexe == 0)
				OpCall_put(&oc, "PP_P_P_", BRG(sub_getarg), 0, 1, (KInt) 0, 2, (KInt) 0, 3, (KInt) 0);
			else {
				qc += kputBin86(&cod[qc], "89_e1  8d_41_14  89_01  8d_41_18  89_41_04");
				qc += kputBin86(&cod[qc], "8d_41_1c  89_41_08  31_c0  89_41_0c");
				qc += kputBin86(&cod[qc], "8d_41_20  89_41_10  ff_95_%i32", DLL___GETMAINARGS);
				KVarArea_addInt(&env->mkexe_dll, &cod[qc] - env->code);
				KInt i;
				for (i = 0; i < 4; i++) {
					if (ps[i * 2] >= 0) {
						qc += kputBin86(&cod[qc], "89_e1  8b_41_14  83_f8_%i8  76_1c  8b_41_18", i + 1);
						qc += kputBin86(&cod[qc], "8b_40_%i8  89_01  31_c0  89_41_04", (i + 1) * 4);
						qc += kputBin86(&cod[qc], "89_41_08  ff_95_%i32", DLL_STRTOL);
						KVarArea_addInt(&env->mkexe_dll, &cod[qc] - env->code);
						qc += kputBin86(&cod[qc], "89_85_%i32", getVarOfs(le[ps[i * 2]].i));	// jbê%m0ɂ͂Ȃ.
					}
				}
			}
		} else if (PC(phr, 313, "@ ARY STATIC INT NEW @t:[@imm];")) {
#if 0
			if (env->mode_mkexe == 0)
				OpCall_put(&oc, "P1c", BRG(sub_aryintnew), 0, (KInt) 1);
			else {
				KInt i = getVarOfs(le[ps[0]].i) + (0x80 + 0x24);
				KInt j = env->mkexe_idt.s;
				KVarArea_reserve1(&env->mkexe_idt, 3 * sizeof (KInt));
				KInt k = getConstInt(le[ps[2]].i);
				KInt l = env->mkexe_idt.s;
				KVarArea_reserve1C1(&env->mkexe_idt, (k + sizeof (KInt)) & - sizeof (KInt));
				put32((unsigned char *) &env->mkexe_idt.p[i], j + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, i << 4 | 5);
				put32((unsigned char *) &env->mkexe_idt.p[j + 0 * sizeof (KInt)], k);
				put32((unsigned char *) &env->mkexe_idt.p[j + 1 * sizeof (KInt)], k);
				put32((unsigned char *) &env->mkexe_idt.p[j + 2 * sizeof (KInt)], l + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, (j + 2 * sizeof (KInt)) << 4 | 5);
			}
#endif
		} else if (PC(phr, 314, "@ ARY STATIC INT @t:[@imm...]=@cs;")) {
#if 0
			if (env->mode_mkexe == 0) {
				qc += loadReg(R02, &cod[qc], sid, le, &ps[2]);
				qc += loadReg(R00, &cod[qc], sid, le, &ps[0]);
				qc += kputBin86(&cod[qc], "%rx1_c1_%mrm_%i8", 3, 4, R02, kpopCount32(sizeof (char) - 1));
				if (env->lwait >= 0 && envl->ndbg == 0) {
					qc += kputBin86(&cod[qc], "%rx1_8b_%mrm_%i8", 1, R01, R00, (int) sizeof (KInt));
					qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R01, R00, (int) (2 * sizeof (KInt)));
				}
				qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R02, R00, (int) (2 * sizeof (KInt)));
				const char *t = KStringId_getStr(sid, le[ps[4]].i)->p;
				int l, m; for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
				t++;
				for (m = 0; m <= l; m++) {
					if (m < l)
						qc += loadRegImm(R00, &cod[qc], t[m]);
					else
						qc += loadRegImm(R00, &cod[qc], 0);
					if (env->lwait >= 0 && envl->ndbg == 0) {
						qc += kputBin86(&cod[qc], "%rx1_3b_%mrm", 3, R02, R01);
						qc += execTimeError(&cod[qc], 0x72);
					}
					qc += kputBin86(&cod[qc], "%rx0_88_%mrm_00", 1, R00, R02); // MOV [RDX],AL.
					qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 0, R02, (int) sizeof (char));	// ADD RDX,sizeof (KInt).
				}
			} else {
				KInt i = getVarOfs(le[ps[0]].i) + (0x80 + 0x24), j, k;
				j = *(int*)&env->mkexe_idt.p[i] - 0x00401000;
				KInt n = *(int*)&env->mkexe_idt.p[j + 2 * sizeof (KInt)] - 0x00401000;
				const unsigned char *t = (const unsigned char *) KStringId_getStr(sid, le[ps[4]].i)->p;
				KInt l, m = getConstInt(le[ps[2]].i);
				for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
				t++;
				for (k = 0; k < l; k++)
					*(unsigned char*)&env->mkexe_idt.p[n + (m + k) * sizeof (char)] = t[k];
			}
#endif
		} else if (PC(phr, 315, "@ @t:[@ex]=@ex;")) {
			qc += loadReg(R02, &cod[qc], sid, le, &ps[2]);
			qc += loadReg0(R00, le[pc].i, &cod[qc]);
			qc += kputBin86(&cod[qc], "%rx1_c1_%mrm_%i8", 3, 4, R02, kpopCount32(sizeof (char) - 1));
			if (env->lwait >= 0 && envl->ndbg == 0) {
				qc += kputBin86(&cod[qc], "%rx1_3b_%mrm", 0, R02, R00);
				qc += execTimeError(&cod[qc], 0x72);
			}
			qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R02, R00, (int) (2 * sizeof (KInt)));
			qc += loadReg(R00, &cod[qc], sid, le, &ps[4]);
			qc += kputBin86(&cod[qc], "%rx0_88_%mrm_00", 1, R00, R02);
		} else if (PC(phr, 316, "@ ARY STATIC CONST INT NEW @t:[@imm];")) {
			if (env->mode_mkexe == 0)
				OpCall_put(&oc, "P1c", BRG(sub_aryintnew), 0, (KInt) sizeof (char));
			else {
				KInt u = sizeof (char);
				KInt i = getVarOfs(le[ps[0]].i);
				while ((env->mkexe_txt.s & (sizeof (KInt) - 1)) != 0)
					KVarArea_addChar(&env->mkexe_txt, 0);
				KInt j = env->mkexe_txt.s;
				KVarArea_reserve1(&env->mkexe_txt, 3 * sizeof (KInt));
				KInt k = getConstInt(le[ps[2]].i);
				KInt l = env->mkexe_txt.s;
				KVarArea_reserve1C1(&env->mkexe_txt, k * u);
				while ((env->mkexe_txt.s & (sizeof (KInt) - 1)) != 0)
					KVarArea_addChar(&env->mkexe_txt, 0);
				qc += kputBin86(&cod[qc], "%i8_%i32", 0xb8 | R00, j + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, (&cod[qc] - env->code - 4) << 4 | 8);	// codeɂ, _txtw.
				qc += kputBin86(&cod[qc], "89_%m0", R00, R05, i);
				put32((unsigned char *) &env->mkexe_txt.p[j + 0 * sizeof (KInt)], k * u);
				put32((unsigned char *) &env->mkexe_txt.p[j + 1 * sizeof (KInt)], k * u);
				put32((unsigned char *) &env->mkexe_txt.p[j + 2 * sizeof (KInt)], l + 0x00401000);
				KVarArea_addInt(&env->mkexe_lnk, (j + 2 * sizeof (KInt)) << 4 | 10);	// _txtɂ, _txtw.
				envl->p316_l = l;
			}
		} else if (PC(phr, 317, "@ ARY STATIC CONST INT @t:[@imm...]=@cs;")) {
			if (env->mode_mkexe == 0) {
				qc += loadReg(R02, &cod[qc], sid, le, &ps[2]);
				qc += loadReg(R00, &cod[qc], sid, le, &ps[0]);
				qc += kputBin86(&cod[qc], "%rx1_c1_%mrm_%i8", 3, 4, R02, kpopCount32(sizeof (char) - 1));
				if (env->lwait >= 0 && envl->ndbg == 0) {
					qc += kputBin86(&cod[qc], "%rx1_8b_%mrm_%i8", 1, R01, R00, (int) sizeof (KInt));
					qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R01, R00, (int) (2 * sizeof (KInt)));
				}
				qc += kputBin86(&cod[qc], "%rx1_03_%mrm_%i8", 1, R02, R00, (int) (2 * sizeof (KInt)));
				const char *t = KStringId_getStr(sid, le[ps[4]].i)->p;
				int l, m; for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
				t++;
				for (m = 0; m <= l; m++) {
					if (m < l)
						qc += loadRegImm(R00, &cod[qc], t[m]);
					else
						qc += loadRegImm(R00, &cod[qc], 0);
					if (env->lwait >= 0 && envl->ndbg == 0) {
						qc += kputBin86(&cod[qc], "%rx1_3b_%mrm", 3, R02, R01);
						qc += execTimeError(&cod[qc], 0x72);
					}
					qc += kputBin86(&cod[qc], "%rx0_88_%mrm_00", 1, R00, R02); // MOV [RDX],AL.
					qc += kputBin86(&cod[qc], "%rx1_83_%mrm_%i8", 3, 0, R02, (int) sizeof (char));	// ADD RDX,sizeof (KInt).
				}
			} else {
				KInt k, n = envl->p316_l;
				const unsigned char *t = (const unsigned char *) KStringId_getStr(sid, le[ps[4]].i)->p;
				KInt l, m = getConstInt(le[ps[2]].i);
				for (l = 0; t[l + 1] != 0 && t[l + 1] != t[0]; l++);
				t++;
				for (k = 0; k < l; k++)
					*(unsigned char*)&env->mkexe_txt.p[n + (m + k) * sizeof (char)] = t[k];
			}

		} else if (PC(phr, 530, "@ @#504 @rm8_,@r8;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "%i8_%rmr", 0x00 | ps[1] << 3); // @#504 (ADD OR ADC SBB AND SUB XOR CMP)
		} else if (PC(phr, 531, "@ @#504 @rm16_,@r16;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_%i8_%rmr", 0x01 | ps[1] << 3);
		} else if (PC(phr, 532, "@ @#504 @rm32_,@r32;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "%i8_%rmr", 0x01 | ps[1] << 3);
		} else if (PC(phr, 559, "@ @#504 @rm64_,@r64;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_%i8_%rmr", 0x01 | ps[1] << 3);
		} else if (PC(phr, 533, "@ @#504 @r8,@rm8_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "%i8_%rmr", 0x02 | ps[1] << 3);
		} else if (PC(phr, 534, "@ @#504 @r16,@rm16_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_%i8_%rmr", 0x03 | ps[1] << 3);
		} else if (PC(phr, 535, "@ @#504 @r32,@rm32_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "%i8_%rmr", 0x03 | ps[1] << 3);
		} else if (PC(phr, 560, "@ @#504 @r64,@rm64_;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_%i8_%rmr", 0x03 | ps[1] << 3);
		} else if (PC(phr, 536, "@ @#504 AL,@imm;")) {				qc += kputBin86(&cod[qc],      "%i8_%i8",  0x04 | ps[1] << 3, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 537, "@ @#504 AX,@imm;")) {				qc += kputBin86(&cod[qc],   "66_%i8_%i16", 0x05 | ps[1] << 3, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 538, "@ @#504 EAX,@imm;")) {				qc += kputBin86(&cod[qc],      "%i8_%i32", 0x05 | ps[1] << 3, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 561, "@ @#504 RAX,@imm;")) {				qc += kputBin86(&cod[qc], "%rx1_%i8_%i32", 0x05 | ps[1] << 3, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 539, "@ A_PUSH ES;")) {					cod[qc] = 0x06; qc++;
		} else if (PC(phr, 540, "@ A_POP ES;")) {					cod[qc] = 0x07; qc++;
		} else if (PC(phr, 541, "@ A_PUSH CS;")) {					cod[qc] = 0x0e; qc++;
		} else if (PC(phr, 542, "@ A_PUSH SS;")) {					cod[qc] = 0x16; qc++;
		} else if (PC(phr, 543, "@ A_POP SS;")) {					cod[qc] = 0x17; qc++;
		} else if (PC(phr, 544, "@ A_PUSH DS;")) {					cod[qc] = 0x1e; qc++;
		} else if (PC(phr, 545, "@ A_POP DS;")) {					cod[qc] = 0x1f; qc++;
//		} else if (PC(phr, 546, "@ ES:")) {							qc += kputBin86(&cod[qc], "26");
		} else if (PC(phr, 547, "@ DAA;")) {						cod[qc] = 0x27; qc++;
//		} else if (PC(phr, 548, "@ CS:")) {							qc += kputBin86(&cod[qc], "2e");
		} else if (PC(phr, 549, "@ DAS;")) {						cod[qc] = 0x2f; qc++;
//		} else if (PC(phr, 550, "@ SS:")) {							qc += kputBin86(&cod[qc], "36");
		} else if (PC(phr, 551, "@ AAA;")) {						cod[qc] = 0x37; qc++;
//		} else if (PC(phr, 552, "@ DS:")) {							qc += kputBin86(&cod[qc], "3e");
		} else if (PC(phr, 553, "@ AAS;")) {						cod[qc] = 0x3f; qc++;
		} else if (PC(phr, 554, "@ A_PUSH @#502;")) {				cod[qc] = 0x50 | ps[1]; qc++;
		} else if (PC(phr, 555, "@ A_POP @#502;")) {				cod[qc] = 0x58 | ps[1]; qc++;
		} else if (PC(phr, 562, "@ A_PUSH @#505;")) {				cod[qc] = 0x50 | ps[1]; qc++;
		} else if (PC(phr, 563, "@ A_POP @#505;")) {				cod[qc] = 0x58 | ps[1]; qc++;
		} else if (PC(phr, 564, "@ A_PUSH @#506;")) {				cod[qc] = 0x41; cod[qc + 1] = 0x50 | ps[1]; qc += 2;
		} else if (PC(phr, 565, "@ A_POP @#506;")) {				cod[qc] = 0x41; cod[qc + 1] = 0x58 | ps[1]; qc += 2;
//		} else if (PC(phr, 556, "@ PUSHAD;")) {						cod[qc] = 0x60; qc++;
//		} else if (PC(phr, 557, "@ POPAD;")) {						cod[qc] = 0x61; qc++;
		} else if (PC(phr, 558, "@ A_PUSH @imm;")) {				qc += kputBin86(&cod[qc], "68_%i32", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 566, "@ IMUL @r16,@rm16_,@imm;")) {		qc += kputBin86x(&cod[qc], pBx, 0,   "66_69_%rmr_%i16", getConstInt(le[ps[4]].i));
		} else if (PC(phr, 567, "@ IMUL @r32,@rm32_,@imm;")) {		qc += kputBin86x(&cod[qc], pBx, 0,      "69_%rmr_%i32", getConstInt(le[ps[4]].i));
		} else if (PC(phr, 568, "@ IMUL @r64,@rm64_,@imm;")) {		qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_69_%rmr_%i32", getConstInt(le[ps[4]].i));
		} else if (PC(phr, 569, "@ A_PUSH BYTE @imm;")) {			qc += kputBin86(&cod[qc], "6a_%i8", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 570, "@ IMUL @r16,@rm16_,BYTE @imm;")) {	qc += kputBin86x(&cod[qc], pBx, 0,   "66_6b_%rmr_%i8", getConstInt(le[ps[4]].i));
		} else if (PC(phr, 571, "@ IMUL @r32,@rm32_,BYTE @imm;")) {	qc += kputBin86x(&cod[qc], pBx, 0,      "6b_%rmr_%i8", getConstInt(le[ps[4]].i));
		} else if (PC(phr, 572, "@ IMUL @r64,@rm64_,BYTE @imm;")) {	qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_6b_%rmr_%i8", getConstInt(le[ps[4]].i));
		} else if (PC(phr, 573, "@ INSB;")) {						cod[qc] = 0x6c; qc++;
		} else if (PC(phr, 574, "@ INSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0x6d; qc += 2;
		} else if (PC(phr, 575, "@ INSD;")) {						cod[qc] = 0x6d; qc++;
		} else if (PC(phr, 576, "@ INSQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0x6d; qc += 2;
		} else if (PC(phr, 577, "@ OUTSB;")) {						cod[qc] = 0x6e; qc++;
		} else if (PC(phr, 578, "@ OUTSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0x6f; qc += 2;
		} else if (PC(phr, 579, "@ OUTSD;")) {						cod[qc] = 0x6f; qc++;
		} else if (PC(phr, 580, "@ OUTSQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0x6f; qc += 2;
		} else if (PC(phr, 581, "@ @#507 @t;") != 0 && envl->progselm >= 0) {
			qc += kputBin86(&cod[qc], "0f_%i8_%r32", 0x80 | ps[1], 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[2]].i));
		} else if (PC(phr, 582, "@ @#508 @t;") != 0 && envl->progselm >= 0) {
			qc += kputBin86(&cod[qc], "0f_%i8_%r32", 0x80 | ps[1], 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[2]].i));
		} else if (PC(phr, 583, "@ JAE @t;") != 0 && envl->progselm >= 0) {
			qc += kputBin86(&cod[qc], "0f_83_%r32", 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[0]].i));
		} else if (PC(phr, 584, "@ @#504 @rm8,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "80_%rmc_%i8",  ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 585, "@ @#504 @rm16,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_81_%rmc_%i16", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 586, "@ @#504 @rm32,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "81_%rmc_%i32", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 587, "@ @#504 @rm64,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_81_%rmc_%i32", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 588, "@ @#504 @rm16,BYTE @imm;")) {		qc += kputBin86x(&cod[qc], pBx, 0,   "66_83_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 589, "@ @#504 @rm32,BYTE @imm;")) {		qc += kputBin86x(&cod[qc], pBx, 0,      "83_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 590, "@ @#504 @rm64,BYTE @imm;")) {		qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_83_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 591, "@ TEST @r8,@rm8_;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "84_%rmr");
		} else if (PC(phr, 592, "@ TEST @r16,@rm16_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_85_%rmr");
		} else if (PC(phr, 593, "@ TEST @r32,@rm32_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "85_%rmr");
		} else if (PC(phr, 594, "@ TEST @r64,@rm64_;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_85_%rmr");
		} else if (PC(phr, 595, "@ XCHG AX,@#501;")) {				cod[qc] = 0x66; cod[qc + 1] = 0x90 | ps[1]; qc += 2;
		} else if (PC(phr, 596, "@ XCHG EAX,@#502;")) {				cod[qc] = 0x90 | ps[1]; qc++;
		} else if (PC(phr, 597, "@ XCHG RAX,@#503;")) {				cod[qc] = 0x48 | ps[1] >> 3; cod[qc + 1] = 0x90 | (ps[1] & 7); qc += 2;
		} else if (PC(phr, 598, "@ XCHG @r8,@rm8_;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "86_%rmr");
		} else if (PC(phr, 599, "@ XCHG @r16,@rm16_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_87_%rmr");
		} else if (PC(phr, 600, "@ XCHG @r32,@rm32_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "87_%rmr");
		} else if (PC(phr, 601, "@ XCHG @r64,@rm64_;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_87_%rmr");
		} else if (PC(phr, 607, "@ MOV @rm8_,@r8;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "88_%rmr");
		} else if (PC(phr, 608, "@ MOV @rm16_,@r16;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_89_%rmr");
		} else if (PC(phr, 609, "@ MOV @rm32_,@r32;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "89_%rmr");
		} else if (PC(phr, 610, "@ MOV @rm64_,@r64;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_89_%rmr");
		} else if (PC(phr, 611, "@ MOV @r8,@rm8_;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "8a_%rmr");
		} else if (PC(phr, 612, "@ MOV @r16,@rm16_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_8b_%rmr");
		} else if (PC(phr, 613, "@ MOV @r32,@rm32_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "8b_%rmr");
		} else if (PC(phr, 614, "@ MOV @r64,@rm64_;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_8b_%rmr");
		} else if (PC(phr, 616, "@ LEA @r16,@rm16_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_8d_%rmr");
		} else if (PC(phr, 617, "@ LEA @r32,@rm32_;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "8d_%rmr");
		} else if (PC(phr, 618, "@ LEA @r64,@rm64_;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_8d_%rmr");
		} else if (PC(phr, 620, "@ A_POP @rm16;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_8f_%rmc", 0);
		} else if (PC(phr, 621, "@ A_POP @rm32;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "8f_%rmc", 0);
		} else if (PC(phr, 622, "@ A_POP @rm64;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_8f_%rmc", 0);
		} else if (PC(phr, 623, "@ NOP;")) {						cod[qc] = 0x90; qc++;
		} else if (PC(phr, 624, "@ CBW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0x98; qc += 2;
		} else if (PC(phr, 625, "@ CWDE;")) {						cod[qc] = 0x98; qc++;
		} else if (PC(phr, 626, "@ CWD;")) {						cod[qc] = 0x66; cod[qc + 1] = 0x99; qc += 2;
		} else if (PC(phr, 627, "@ CDQ;")) {						cod[qc] = 0x99; qc++;
		} else if (PC(phr, 630, "@ PUSHF;")) {						cod[qc] = 0x9c; qc++;
		} else if (PC(phr, 631, "@ PUSHFD;")) {						cod[qc] = 0x9c; qc++;
		} else if (PC(phr, 632, "@ POPF;")) {						cod[qc] = 0x9d; qc++;
		} else if (PC(phr, 633, "@ POPFD;")) {						cod[qc] = 0x9d; qc++;
		} else if (PC(phr, 634, "@ SAHF;")) {						cod[qc] = 0x9e; qc++;
		} else if (PC(phr, 635, "@ LAHF;")) {						cod[qc] = 0x9f; qc++;
		} else if (PC(phr, 636, "@ MOVSB;")) {						cod[qc] = 0xa4; qc++;
		} else if (PC(phr, 637, "@ MOVSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0xa5; qc += 2;
		} else if (PC(phr, 638, "@ MOVSD;")) {						cod[qc] = 0xa5; qc++;
		} else if (PC(phr, 639, "@ MOVSQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0xa5; qc += 2;
		} else if (PC(phr, 640, "@ CMPSB;")) {						cod[qc] = 0xa6; qc++;
		} else if (PC(phr, 641, "@ CMPSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0xa7; qc += 2;
		} else if (PC(phr, 642, "@ CMPSD;")) {						cod[qc] = 0xa7; qc++;
		} else if (PC(phr, 643, "@ CMPSQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0xa7; qc += 2;
		} else if (PC(phr, 644, "@ TEST AL,@imm;")) {				qc += kputBin86(&cod[qc],      "a8_%i8", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 645, "@ TEST AX,@imm;")) {				qc += kputBin86(&cod[qc],   "66_a9_%i16", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 646, "@ TEST EAX,@imm;")) {				qc += kputBin86(&cod[qc],      "a9_%i32", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 647, "@ TEST RAX,@imm;")) {				qc += kputBin86(&cod[qc], "%rx1_a9_%i32", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 648, "@ STOSB;")) {						cod[qc] = 0xaa; qc++;
		} else if (PC(phr, 649, "@ STOSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0xab; qc += 2;
		} else if (PC(phr, 650, "@ STOSD;")) {						cod[qc] = 0xab; qc++;
		} else if (PC(phr, 651, "@ STOSQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0xab; qc += 2;
		} else if (PC(phr, 652, "@ LODSB;")) {						cod[qc] = 0xac; qc++;
		} else if (PC(phr, 653, "@ LODSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0xad; qc += 2;
		} else if (PC(phr, 654, "@ LODSD;")) {						cod[qc] = 0xad; qc++;
		} else if (PC(phr, 655, "@ LODSQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0xad; qc += 2;
		} else if (PC(phr, 656, "@ SCASB;")) {						cod[qc] = 0xae; qc++;
		} else if (PC(phr, 657, "@ SCADSW;")) {						cod[qc] = 0x66; cod[qc + 1] = 0xaf; qc += 2;
		} else if (PC(phr, 658, "@ SCASD;")) {						cod[qc] = 0xaf; qc++;
		} else if (PC(phr, 659, "@ SCASQ;")) {						cod[qc] = 0x48; cod[qc + 1] = 0xaf; qc += 2;
		} else if (PC(phr, 602, "@ MOV @#500,@imm;")) {				cod[qc] = 0xb0 | ps[1]; cod[qc + 1] = getConstInt(le[ps[2]].i); qc += 2;
		} else if (PC(phr, 603, "@ MOV @#501,@imm;")) {				qc += kputBin86(&cod[qc], "66_%i8_%i16", 0xb8 | ps[1], getConstInt(le[ps[2]].i));
		} else if (PC(phr, 604, "@ MOV @#502,@imm;")) {				qc += kputBin86(&cod[qc],    "%i8_%i32", 0xb8 | ps[1], getConstInt(le[ps[2]].i));
		} else if (PC(phr, 605, "@ MOV @#505,@imm;")) {
			KInt i = getConstInt(le[ps[2]].i);
			if (0 <= i && i <= 0xffffffffLL)
				qc += kputBin86(&cod[qc], "%i8_%i32", 0xb8 | ps[1], i);
			else
				qc += kputBin86(&cod[qc], "48_%i8_%i64", 0xb8 | ps[1], i);
		} else if (PC(phr, 606, "@ MOV @#506,@imm;")) {
			KInt i = getConstInt(le[ps[2]].i);
			if (0 <= i && i <= 0xffffffffLL)
				qc += kputBin86(&cod[qc], "41_%i8_%i32", 0xb8 | ps[1], i);
			else
				qc += kputBin86(&cod[qc], "49_%i8_%i64", 0xb8 | ps[1], i);
		} else if (PC(phr, 684, "@ SAL @rm8,1;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "d0_%rmc", 4);
		} else if (PC(phr, 685, "@ @#509 @rm8,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d0_%rmc", ps[1]);
		} else if (PC(phr, 686, "@ SAL @rm16,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_d1_%rmc", 4);
		} else if (PC(phr, 687, "@ @#509 @rm16,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_d1_%rmc", ps[1]);
		} else if (PC(phr, 688, "@ SAL @rm32,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d1_%rmc", 4);
		} else if (PC(phr, 689, "@ @#509 @rm32,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d1_%rmc", ps[1]);
		} else if (PC(phr, 690, "@ SAL @rm64,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_d1_%rmc", 4);
		} else if (PC(phr, 691, "@ @#509 @rm64,1;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_d1_%rmc", ps[1]);
		} else if (PC(phr, 660, "@ SAL @rm8,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "c0_%rmc_%i8", 4,     getConstInt(le[ps[2]].i));
		} else if (PC(phr, 661, "@ @#509 @rm8,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "c0_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 662, "@ SAL @rm16,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_c1_%rmc_%i8", 4,     getConstInt(le[ps[2]].i));
		} else if (PC(phr, 663, "@ @#509 @rm16,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_c1_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 664, "@ SAL @rm32,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "c1_%rmc_%i8", 4,     getConstInt(le[ps[2]].i));
		} else if (PC(phr, 665, "@ @#509 @rm32,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "c1_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 666, "@ SAL @rm64,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_c1_%rmc_%i8", 4,     getConstInt(le[ps[2]].i));
		} else if (PC(phr, 667, "@ @#509 @rm64,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_c1_%rmc_%i8", ps[1], getConstInt(le[ps[4]].i));
		} else if (PC(phr, 668, "@ RET @imm;")) {					qc += kputBin86(&cod[qc], "c2_%i16", getConstInt(le[ps[0]].i));
		} else if (PC(phr, 669, "@ RET;")) {						cod[qc] = 0xc3; qc++;
		} else if (PC(phr, 672, "@ MOV @rm8,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "c6_%rmc_%i8",  0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 673, "@ MOV @rm16,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_c7_%rmc_%i16", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 674, "@ MOV @rm32,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "c7_%rmc_%i32", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 675, "@ MOV @rm64,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_c7_%rmc_%i32", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 692, "@ SAL @rm8,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d2_%rmc", 4);
		} else if (PC(phr, 693, "@ @#509 @rm8,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d2_%rmc", ps[1]);
		} else if (PC(phr, 694, "@ SAL @rm16,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_d3_%rmc", 4);
		} else if (PC(phr, 695, "@ @#509 @rm16,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0,   "66_d3_%rmc", ps[1]);
		} else if (PC(phr, 696, "@ SAL @rm32,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d3_%rmc", 4);
		} else if (PC(phr, 697, "@ @#509 @rm32,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "d3_%rmc", ps[1]);
		} else if (PC(phr, 698, "@ SAL @rm64,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_d3_%rmc", 4);
		} else if (PC(phr, 699, "@ @#509 @rm64,CL;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_d3_%rmc", ps[1]);

		} else if (PC(phr, 700, "@ CALL @t;")) {
			qc += kputBin86(&cod[qc], "e8_%r32", 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[0]].i));
		} else if (PC(phr, 701, "@ JMP @t;")) {
			qc += kputBin86(&cod[qc], "e9_%r32", 0);
			KVarArea_addInt(&envl->gotoLink, qc - 4);
			KVarArea_addInt(&envl->gotoLink, labelId(le[ps[0]].i));
		} else if (PC(phr, 702, "@ TEST @rm8,@imm;")) {				qc += kputBin86x(&cod[qc], pBx, 0,      "f6_%rmc_%i8", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 703, "@ NOT @rm8;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "f6_%rmc", 2);
		} else if (PC(phr, 704, "@ NEG @rm8;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "f6_%rmc", 3);
		} else if (PC(phr, 705, "@ TEST @rm16,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,   "66_f7_%rmc_%i16", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 706, "@ TEST @rm32,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0,      "f7_%rmc_%i32", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 707, "@ TEST @rm64,@imm;")) {			qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_f7_%rmc_%i32", 0, getConstInt(le[ps[2]].i));
		} else if (PC(phr, 708, "@ NOT @rm16;")) {					qc += kputBin86x(&cod[qc], pBx, 0,   "66_f7_%rmc", 2);
		} else if (PC(phr, 709, "@ NOT @rm32;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "f7_%rmc", 2);
		} else if (PC(phr, 710, "@ NOT @rm64;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_f7_%rmc", 2);
		} else if (PC(phr, 711, "@ NEG @rm16;")) {					qc += kputBin86x(&cod[qc], pBx, 0,   "66_f7_%rmc", 3);
		} else if (PC(phr, 712, "@ NEG @rm32;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "f7_%rmc", 3);
		} else if (PC(phr, 713, "@ NEG @rm64;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_f7_%rmc", 3);
		} else if (PC(phr, 714, "@ INC @rm8;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "fe_%rmc", 0);
		} else if (PC(phr, 715, "@ DEC @rm8;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "fe_%rmc", 1);
		} else if (PC(phr, 716, "@ INC @rm16;")) {					qc += kputBin86x(&cod[qc], pBx, 0,   "66_ff_%rmc", 0);
		} else if (PC(phr, 717, "@ INC @rm32;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "ff_%rmc", 0);
		} else if (PC(phr, 718, "@ INC @rm64;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_ff_%rmc", 0);
		} else if (PC(phr, 719, "@ DEC @rm16;")) {					qc += kputBin86x(&cod[qc], pBx, 0,   "66_ff_%rmc", 1);
		} else if (PC(phr, 720, "@ DEC @rm32;")) {					qc += kputBin86x(&cod[qc], pBx, 0,      "ff_%rmc", 1);
		} else if (PC(phr, 721, "@ DEC @rm64;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "%rx1_ff_%rmc", 1);
		} else if (PC(phr, 722, "@ CALL @rm32;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "ff_%rmc", 2);
		} else if (PC(phr, 723, "@ CALL @rm64;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "ff_%rmc", 2);
		} else if (PC(phr, 724, "@ JMP @rm32;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "ff_%rmc", 4);
		} else if (PC(phr, 725, "@ JMP @rm64;")) {					qc += kputBin86x(&cod[qc], pBx, 0, "ff_%rmc", 4);
		} else if (PC(phr, 726, "@ A_PUSH @rm32;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "ff_%rmc", 6);
		} else if (PC(phr, 727, "@ A_PUSH @rm64;")) {				qc += kputBin86x(&cod[qc], pBx, 0, "ff_%rmc", 6);








		} else if (PC(phr,  10, "@ CODE")) {
			KInt lpc = pc + 1; //, eax = lx->sid->getId0("EAX"), ttt = 0;
			for (;;) {
				phr->prI0 = lpc;
				if (PC(phr,  11, "@ ;")) break;
				if (testConstInt(le[lpc].i) != 0) {	cod[qc++] = getConstInt(le[lpc].i); lpc++; continue; }
				if (PC(phr,  12, "@ % @imm")) {		qc += kputBin86(&cod[qc], "%i32", getConstInt(le[lpc + 1].i)); lpc += 2; continue; }
				if (PC(phr,  13, "@ %% @imm")) {		qc += kputBin86(&cod[qc], "%i16", getConstInt(le[lpc + 1].i)); lpc += 2; continue; }
#if 0
				if (phr->cmpPr( 88, "@ #RM : @* , ")) {
					ttt = lx->getConstInt(le[lpc + 2].i);
					if (eax <= le[lpc + 2].i && le[lpc + 2].i < eax + 24)
						ttt = (le[lpc + 2].i - eax) & 7;
				}
				if (phr->cmpPr( 89, "@ #RM : @* , [ @* ]")) {
					if (eax <= le[lpc + 5].i && le[lpc + 5].i < eax + 8) {
						if (le[lpc + 5].i == eax + ESP) {
							qc += kputBin(&cod[qc], "%i8_24", ttt << 3 | ESP);
						} else if (le[lpc + 5].i == eax + EBP) {
							qc += kputBin(&cod[qc], "%i8_00", ttt << 3 | 0x40 | EBP);
						} else
							cod[qc++] = ttt << 3 | (le[lpc + 5].i - eax);
					} else
						qc += kputBin(&cod[qc], "%i8_%i32", ttt << 3 | 0x85, le[lpc + 5].i * 4);
					lpc += 7;
					continue;
				}
				if (phr->cmpPr( 90, "@ #RM : @* , [ @* + @* ]")) {
					if (eax <= le[lpc + 5].i && le[lpc + 5].i <= eax + 7) {
						int disp = lx->getConstInt(le[lpc + 7].i);
						if (-128 <= disp && disp <= 127) {
							if (le[lpc + 5].i == eax + ESP) {
								qc += kputBin(&cod[qc], "%i8_24_%i8", ttt << 3 | 0x40 | ESP, disp);
							} else
								qc += kputBin(&cod[qc], "%i8_%i8", ttt << 3 | 0x40 | (le[lpc + 5].i - eax), disp);
						} else {
							if (le[lpc + 5].i == eax + ESP) {
								qc += kputBin(&cod[qc], "%i8_24_%i32", ttt << 3 | 0x80 | ESP, disp);
							} else
								qc += kputBin(&cod[qc], "%i8_%i32", ttt << 3 | 0x80 | (le[lpc + 5].i - eax), disp);
						}
					} else
						goto err;
					lpc += 9;
					continue;
				}
				if (phr->cmpPr( 91, "@ #RM : @* , @*")) {
					cod[qc++] = ttt << 3 | 0xc0 | ((le[lpc + 4].i - eax) & 7);
					lpc += 5;
					continue;
				}
				if (phr->cmpPr( 92, "@ #REL : @*")) {
					qc += kputBin(&cod[qc], "%r32", 0);
					int i = ~le[lpc + 2].i;
					if (lx->testConstInt(le[lpc + 2].i))
						i = lx->getConstInt(le[lpc + 2].i);
					envl->gotoLink.addInt(qc - 4)->addInt(i);
					lpc += 3;
					continue;
				}
				if (phr->cmpPr( 93, "@ #OR : @* , @*")) {
					int i = lx->getConstInt(le[lpc + 2].i), j = lx->getConstInt(le[lpc + 4].i);
					if (eax <= le[lpc + 2].i && le[lpc + 2].i < eax + 24)
						i = (le[lpc + 2].i - eax) & 7;
					if (eax <= le[lpc + 4].i && le[lpc + 4].i < eax + 24)
						j = (le[lpc + 4].i - eax) & 7;
					cod[qc++] = i | j;
					lpc += 5;
					continue;
				}
#endif
				goto err;
			}




		} else
			goto err;
		while (le[pc].i != semi)
			pc++;
#if 0
		if (xl.s > 0) {
			pc++;
			oc.le = le = (KLexer_Elmt *) lx->sp->reserve(lx->sp->s + xl.s)->p;
			lx->sp->s += xl.s;
			memmove(((char *) &le[pc]) + xl.s, &le[pc], ((char *) &le[pc1]) - ((char *) &le[pc]));
			memcpy(&le[pc], xl.p, xl.s);
			pc1 = lx->getElmtLen();
			xl.s = 0;
			pc--;
		}
#endif
	}
	env->qc = qc;
	KVarArea_dest(&xl);
	return 0;
err:
//printf("%d %d\n", (int) le[1].i, (int) phr->code[10 * phr->l0 + 0]);
#if 0
for (i = pc; i < pc1;i++) {
KStringId_Elmt *sss = KStringId_getStr(sid, le[i].i);
printf("%.*s ", (int) sss->l, sss->p);
}
printf("\n'%.*s'\n", (int) l, s);
#endif
	envl->err = 1;
	if (env->ln >= 0)
		printf("syntax error [%d] : %.*s\n", (int) env->ln, (int) kminInt(l, 30), le[pc].p0);
	else
		printf("syntax error : %.*s\n", (int) kminInt(l, 30), le[pc].p0);
	env->qc = qc;
	KVarArea_dest(&xl);
	return 1;
}

void eval1()
{
	*getVarPtr(env->sysLine) = -1;
	if (envl->stkFor.s != 0) {
		printf("nest error : FOR\n");
		return;
	}
	if (envl->stkIf.s != 0) {
		printf("nest error : IF\n");
		return;
	}
	KInt i;
	for (i = 0; i < envl->gotoLink.s; i += 2 * sizeof (KInt)) {
		KInt *p = (KInt *) &envl->gotoLink.p[i];
		KInt *pp = (KInt *) KIndexDict0_search1(&envl->label, &p[1]);
		if (pp != 0)
			kputBin86(&env->code[p[0]], "%r32", (KInt) &env->code[pp[1]]);
		else {
			if (p[1] >= 0)
				fprintf(stderr, "goto warning : %d\n", (int) p[1]);
			else {
				KStringId_Elmt *se = KStringId_getStr(env->lx.sid, ~p[1]);
				fprintf(stderr, "goto warning : %.*s\n", (int) se->l, se->p);
			}
			kputBin86(&env->code[p[0]], "%r32", (KInt) &env->code[env->qc]);
		}
	}
	if (envl->progselm >= 0) {	// _CNgR}h͂ł͂Ȃ. ݂̐ݒp. ̕fobO͕֗낤.
		*getVarPtr(env->sysReg0 + 7) = sub_initstack(1024);	// RDI/EDI.
		*getVarPtr(env->sysReg0 - 2) = 0;	// $DI
	}
	if (env->mode_mkexe == 0) {
		env->qc += kputBin86(&env->code[env->qc], "e8_%r32  e8_%r32  e9_%r32", (KInt) env->saveRegs, (KInt) BRG(sub_end), (KInt) env->exitProg);
		#if (defined(KUSE64))
			kputBin86(&env->code[envl->qc0 + 33], "%i64", (KInt) env->var.p);
		#elif (defined(KUSE32))
			kputBin86(&env->code[envl->qc0 + 5], "%i32", (KInt) env->var.p);
		#endif
	} else {
		env->qc += kputBin86(&env->code[env->qc], "83_c4_78 61 5d c3");
	}
	if (env->codedump == 1) {  //  fobOp.
		int i;
		#if (defined(KUSE64))
			for (i = envl->qc0 + 65; i < env->qc - 15; i++) { printf("%02x ", env->code[i]); }
			printf("\n%dbytes\n", (int) (env->qc - envl->qc0 - (65 + 15)));
		#elif (defined(KUSE32))
			for (i = envl->qc0 + 29; i < env->qc - 15; i++) { printf("%02x ", env->code[i]); }
			printf("\n%dbytes\n", env->qc - envl->qc0 - (29 + 15));
		#endif
	}
	env->lwcount = 0;
	if (env->mode_mkexe == 0)
		((void (*)()) (env->code + envl->qc0))();
}

void eval2()
{
	env->qc = envl->qc0;
	KVarArea_dest(&envl->UsrStk);
	KVarArea_dest(&envl->alias);
	KVarArea_dest(&envl->stkFor);
	KVarArea_dest(&envl->stkIf);
	KVarArea_dest(&envl->gotoLink);
	KVarArea_dest(&envl->cmplTime);
	// sԍ.
	KInt i, j, j1;
	KVarArea **p0 = envl->label.p;
	for (i = 0; i <= envl->label.n1; i++) {
		if (p0[i] != 0) {
			j1 = p0[i]->s / sizeof (KIndexHSNode);
			KIndexHSNode *p2 = (KIndexHSNode *) p0[i]->p;
			for (j = 0; j < j1; j++)
				KMemPool_free(&kmemPool, 2 * sizeof (KInt), p2[j].d);
		}
	}
	KIndexDict0_dest(&envl->label);
	EnvLocal *envl0 = envl->envl0;
	KMemPool_free(&kmemPool, sizeof (EnvLocal), envl);
	envl = envl0;
}

void putKInt(unsigned char *p, KInt i)
{
	p[0] =  i        & 0xff;
	p[1] = (i >>  8) & 0xff;
	p[2] = (i >> 16) & 0xff;
	p[3] = (i >> 24) & 0xff;
	#if (defined(KUSE64))
		p[4] = (i >> 32) & 0xff;
		p[5] = (i >> 40) & 0xff;
		p[6] = (i >> 48) & 0xff;
		p[7] = (i >> 56) & 0xff;
	#endif
}

KInt getKInt(const unsigned char *p)
{
	#if (defined(KUSE64))
		return p[0] | p[1] << 8 | p[2] << 16 | (KInt) p[3] << 24 | (KInt) p[4] << 32 | (KInt) p[5] << 40 | (KInt) p[6] << 48 | (KInt) p[7] << 56;
	#elif (defined(KUSE32))
		return p[0] | p[1] << 8 | p[2] << 16 | p[3] << 24;
	#endif
}

int cmplTimeRun0(KVarArea *va)
{
	KInt psel1, an0 = 0, an1, cd0 = env->codedump;
	env->codedump = 0;
	if (envl != 0)
		an0 = KVarArea_sizInt(&envl->alias);
	sub_prognew(&psel1);
	Prog *pp = prog_getProg(psel1);
	const char *p1 = va->p + va->s, *p = va->p;
	while (p < p1) {
		KInt ln = getKInt((const unsigned char *) p); p += sizeof (KInt);
		KInt l = getKInt((const unsigned char *) p); p += sizeof (KInt);
		KSrc_insLine(&pp->src, ln, l, p); p += l;
	}
	int i = sub_run(psel1);
	an1 =  KVarArea_sizInt(&envl->alias);
	eval2();
	sub_progdel(psel1);
	if (envl != 0) {
		for (; an0 < an1; an0++)
			KVarArea_addInt(&envl->alias, an0);
	}
	env->codedump = cd0;
	return i;
}

int cmplTimeRun1(KVarArea *va)
{
	KInt i = KStringId_getId1L(&env->spCacheKey, va->s, va->p), j;
	if (i * ((int) sizeof (KInt)) < env->spCacheVal.s) {
		j = KVarArea_getIntIdx(&env->spCacheVal, i);
		if ((KUInt) j == 0xdeaddead)
			return 1;
		*getVarPtr(env->sysRetVal) = j;
		return 0;
	}
	int k = cmplTimeRun0(va);
	if (k != 0)
		KVarArea_addInt(&env->spCacheVal, 0xdeaddead);
	else
		KVarArea_addInt(&env->spCacheVal, *getVarPtr(env->sysRetVal));
	return k;
}

int sub_run(KInt psel)
{
	Prog *pp = prog_getProg(psel);
	KIndexS *pn = &pp->src.ln;
	eval0(psel);
	int i, i1 = KVarArea_sizPtr(&pn->va);
	for (i = 0; i < i1; i++) {
		KSrcLine *pl = (KSrcLine *) KVarArea_getPtrIdx(&pn->va, i);
		const char *p = pl->src.p, *p1 = p + pl->src.s;
		if (strncmp(p, "@CMPLTIME", 9) == 0) {
			envl->cmplTimeNest++;
			continue;
		}
		if (strncmp(p, "@EXECTIME", 9) == 0) {
			envl->cmplTimeNest--;
			if (envl->cmplTimeNest == 0) {
				int j = cmplTimeRun1(&envl->cmplTime);
				envl->cmplTime.s = 0;
				if (j != 0) return j;
			}
			if (envl->cmplTimeNest < 0) {
				fprintf(stderr, "@EXECTIME error : %d\n", (int) pl->ln);
				return 1;
			}
			continue;
		}
		if (envl->cmplTimeNest > 0) {
			putKInt(KVarArea_reserve1(&envl->cmplTime, sizeof (KInt)), pl->ln);
			putKInt(KVarArea_reserve1(&envl->cmplTime, sizeof (KInt)), p1 - p);
			KVarArea_addBytes(&envl->cmplTime, p1 - p, p);
			continue;
		}
		p += ksearchToken0((const unsigned char *) p, 0, pl->src.s, env->lx.chrTyp);
		env->ln = pl->ln; // G[\̂߂ɕKv.
		if (KIndexDict0_search1(&envl->label, &pl->ln) == 0) {
			KInt *label = (KInt *) KMemPool_alloc(&kmemPool, 2 * sizeof (KInt));
			label[0] = pl->ln;
			label[1] = env->qc;
			KIndexDict0_ins(&envl->label, label);
		} else
			fprintf(stderr, "label warning : %d\n", (int) pl->ln);
		if (env->lwait >= 0 && psel >= 0 && envl->ndbg == 0) {
			KVarArea *lc = pp->lc;
			env->qc += kputBin86(&env->code[env->qc], "e8_%r32", env->saveRegs);
			#if (defined(KUSE64))
				if ((KInt) (lc->p + (2 * i + 1) * ((int) sizeof (KInt))) <= 0x7fffffff)
					env->qc += kputBin86(&env->code[env->qc], "31_c0   %rx1_83_%mrm_%i32_01   %rx1_83_%mrm_%i32_00", 2, 0, R00, lc->p + (2 * i) * ((int) sizeof (KInt)), 2, 2, R00, lc->p + (2 * i + 1) * ((int) sizeof (KInt)));
				else
					printf("line count warning : %d\n", (int) pl->ln);
			#elif (defined(KUSE32))
				env->qc += kputBin86(&env->code[env->qc], "83_05_%i32_01   83_15_%i32_00", lc->p + (2 * i) * ((int) sizeof (KInt)), lc->p + (2 * i + 1) * ((int) sizeof (KInt)));
			#endif
			env->qc += kputBin86(&env->code[env->qc], "b8_%i32   %rx1_89_%mrm_%i32   e8_%r32", (KInt) pl->ln, 2, R00, R05, getVarOfs(env->sysLine), BRG(sub_lwait_));
			if (pp->dtrp != 0) {
				env->qc += kputBin86(&env->code[env->qc], "%rx1_8b_%mrm_%i32  %rx1_85_%mrm", 2, R00, R05, getVarOfs(env->sysReg0 - 2), 3, R00, R00);
				#if (defined(KUSE64))
					env->qc += kputBin86(&env->code[env->qc], "74_03  %rx1_89_%mrm", 3, R00, R07);
					env->qc += kputBin86(&env->code[env->qc], "%rx1_b8_%i64  %rx1_89_%mrm  %rx1_83_%mrm_%i8  e9_%r32", (KInt) &env->code[env->qc + 22], 0, R00, R07, 3, 0, R07, (int) sizeof (KInt), (KInt) 0);
				#elif (defined(KUSE32))
					env->qc += kputBin86(&env->code[env->qc], "74_02  89_%mrm", 3, R00, R07);
					env->qc += kputBin86(&env->code[env->qc], "b8_%i32  89_07  83_%mrm_%i8  e9_%r32", (KInt) &env->code[env->qc + 15], 3, 0, R07, (int) sizeof (KInt), (KInt) 0);
				#endif
				KVarArea_addInt(&envl->gotoLink, env->qc - 4);
				KVarArea_addInt(&envl->gotoLink, pp->dtrp);
			}
			env->qc += kputBin86(&env->code[env->qc], "e8_%r32", env->loadRegs);
		}
		if (evalAdd(p1 - p, p) != 0) return 1;
	}
	if (envl->cmplTimeNest > 0) {
		fprintf(stderr, "@CMPLTIME error\n");
		return 1;
	}
	eval1();
	return 0;
}

int sub_mkexe(KInt psel, const char *path)
{
	// .textɒuf[^i萔ji[邽߂va, .idataɒuf[^i[邽߂va, .textւ̃|C^w߂̃NC|C^i[pva
	env->mode_mkexe = 1;
	env->mkexe_txt.s = 0;
	env->mkexe_idt.s = 0;
	env->mkexe_lnk.s = 0;
	env->mkexe_dll.s = 0;
	env->mkexe_bss = 0;
	KIndexDict0_reset(&env->mkexe_vofs);	// Ԃ񂱂̂ł͕s\ [mikan]
		// Ԃ炳Ă邷ׂẴm[h񋓂āAR[obNĂāAfree炢Ǝv.
	KVarArea_reserve1C1(&env->mkexe_idt, 4 * 13);
	if (sub_run(psel) != 0)
		return 1;
	KInt i, i1 = KVarArea_sizInt(&env->mkexe_lnk), j, k, *p = (KInt *) env->mkexe_lnk.p;
	if ((env->qc & 1) != 0)
		env->code[env->qc++] = 0;	// 2oCgAC.
	unsigned char *q = (unsigned char *) env->mkexe_idt.p;
	put32(&q[0x0c], env->qc - envl->qc0 + 0x1000 + 32);
	put32(&q[0x24], env->qc - envl->qc0 + 0x1000 +  0);
	put32(&q[0x28], env->qc - envl->qc0 + 0x1000 +  8);
	put32(&q[0x2c], env->qc - envl->qc0 + 0x1000 + 23);
	while ((env->qc & 1) != 0)
		env->code[env->qc++] = 0;
	memcpy(&env->code[env->qc], "\0\0printf\0\0__getmainargs\0\0strtol\0msvcrt.dll", 43);
	env->qc += 43;
	while ((env->qc & 7) != 0)
		env->code[env->qc++] = 0;
	KInt dat0 = (env->qc - envl->qc0 + env->mkexe_txt.s + 0xfff) & -0x1000;
	put32(&q[0x10], dat0 + 0x2000 + 0x24);
	q = &env->code[envl->qc0 + 8];
	put32(q, dat0 + 0x00402000);
	for (i = 0; i < i1; i++) {
		q = env->code;
		k = p[i] & 3;
		j = p[i] >> 4;
		if (k == 1)
			q = (unsigned char *) env->mkexe_idt.p;
		if (k == 2)
			q = (unsigned char *) env->mkexe_txt.p;
		q += j;
		k = q[0] | q[1] << 8 | q[2] << 16 | q[3] << 24;
		if ((p[i] & 0xc) == 0x8)
			k += env->qc - envl->qc0;
		if ((p[i] & 0xc) == 0x4)
			k += dat0;
		put32(q, k);
	}
	memcpy(&env->code[env->qc], env->mkexe_txt.p, env->mkexe_txt.s);
	env->qc += env->mkexe_txt.s;
	unsigned char b[0x200 + 8];
	static unsigned char head[512] = {
		0x4d, 0x5a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 000-00f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 010-01f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 020-02f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, // 030-03f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 040-04f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 050-05f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 060-06f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 070-07f
		0x50, 0x45, 0x00, 0x00, 0x4c, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 080-08f
		0x00, 0x00, 0x00, 0x00, 0xe0, 0x00, 0x03, 0x01, 0x0b, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 090-09f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0a0-0af
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, // 0b0-0bf
		0x04, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0c0-0cf
		0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, // 0d0-0df
		0x00, 0x00, 0x20, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x10, 0x00, 0x00, // 0e0-0ef (heap, stack)
		0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0f0-0ff
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 100-10f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 110-11f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 120-12f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 130-13f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 140-14f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 150-15f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 160-16f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2e, 0x74, 0x65, 0x78, 0x74, 0x00, 0x00, 0x00, // 170-17f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, // 180-18f
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x60, // 190-19f
		0x2e, 0x62, 0x73, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 1a0-1af
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 1b0-1bf
		0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0xc0, 0x2e, 0x69, 0x64, 0x61, 0x74, 0x61, 0x00, 0x00, // 1c0-1cf
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 1d0-1df
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0xc0, // 1e0-1ef
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 1f0-1ff
	};
	memcpy(b, head, 0x200);

#if 1*0
	i = (env->mkexe_idt.s & -0x1000) + 1;
	if ((env->mkexe_idt.s & 0xfff) == 0)
		i = env->mkexe_idt.s - 0xfff;
	while (env->mkexe_idt.s > i && env->mkexe_idt.p[env->mkexe_idt.s - 1] == 0)
		env->mkexe_idt.s--;
#endif

	put32(&b[0x009c], (env->qc - envl->qc0 + 0x1ff) & -0x200);
	put32(&b[0x00a0], env->mkexe_idt.s);
	put32(&b[0x00a4], 0x1000);	// 	Ƃ肠Abss=4KŌŒ.
	put32(&b[0x00d0], dat0 + 0x2000 + ((env->mkexe_idt.s + 0xfff) & -0x1000));
	put32(&b[0x0100], dat0 + 0x2000);
	put32(&b[0x0104], env->mkexe_idt.s);
	put32(&b[0x0180], env->qc - envl->qc0);
	put32(&b[0x0188], (env->qc - envl->qc0 + 0x1ff) & -0x200);
	put32(&b[0x01a8], 0x1000);
	put32(&b[0x01ac], dat0 + 0x1000);
	put32(&b[0x01d0], env->mkexe_idt.s);
	put32(&b[0x01d4], dat0 + 0x2000);
	put32(&b[0x01d8], env->mkexe_idt.s);
	put32(&b[0x01dc], 0x200 + ((env->qc - envl->qc0 + 0x1ff) & -0x200));
	FILE *fp = fopen(path, "wb");
	fwrite(b, 1, 0x200, fp);
	i = env->qc - envl->qc0;
	fwrite(&env->code[envl->qc0], 1, i, fp);
	i &= 0x1ff;
	if (i > 0) {
		memset(b, 0x00, 0x200);
		fwrite(b, 1, 0x200 - i, fp);
	}
	fwrite(env->mkexe_idt.p, 1, env->mkexe_idt.s, fp);
	fclose(fp);
	return 0;
}

void cmdlineLoop(char flg)
{
	KInt bati = 0, i;
	char *p, promptFlag = 1;
	for (;;) {
		es_flushAll(env->gwin);
reloop:
		if (env->batch.s > 0) {
			while (bati < env->batch.s && env->batch.p[bati] <= ' ')
				bati++;
			if (bati >= env->batch.s) {
				env->batch.s = 0;
				bati = 0;
			}
		}
//printf("bati=%d [%.*s] ", (int) bati, strlen(env->cmdlin) - 1, env->cmdlin);
		if (env->batch.s == 0 && promptFlag != 0) {
			if (flg == 0)
				printf("\nOk\n");
			else
				printf("\n(Ok)\n");
		}
		promptFlag = 1;
		if (env->batch.s == 0)
			fgets(env->cmdlin, 65536, stdin);
		else {
			for (i = 0; i + bati < env->batch.s; i++) {
				if (i >= 65532) {
					fprintf(stderr, "bad batch file.\n");
					env->batch.s = 0;
					bati = 0;
					goto reloop;
				}
				env->cmdlin[i] = env->batch.p[i + bati];
				if (env->cmdlin[i] == '\n')
					break;
				if (env->cmdlin[i] == '\r') {
					i--;
					bati++;
				}
			}
			env->cmdlin[i] = '\n';
			env->cmdlin[i + 1] = '\0';
			bati += i + 1;
		}
		clock_t t0 = clock();
		for (p = env->cmdlin; *p == ' ' || *p == '\t'; p++);
		promptFlag = 0;
		if (*p == '\n') goto reloop;
		kupperCaseM(strlen(p), (unsigned char *) p, env->lx.chrTyp);
		if ('0' <= *p && *p <= '9') {
			Prog *pp = prog_getProg(env->progseln);
			KSrc_editLine(&pp->src, strchr(p, '\n') - p, p);
			prog_delLinCnt(pp);
			if (env->batch.s > 0)
				promptFlag = 1;
			goto reloop;
		}
		promptFlag = 1;
//		testKeyBuf();
		env->sysBreak = 0;
		if (strncmp(p, "RUN", 3) == 0 && p[3] <= ' ') {
			env->runParam = (unsigned char *) p;
			if (sub_run(env->progseln) != 0)
				continue;
		} else if (strncmp(p, "MKEXE", 5) == 0 && p[5] <= ' ') {
			for (p += 5; *p == ' ' || *p == '\t'; p++);
			for (i = 0; p[i] >= ' ' && p[i] != ';'; i++);
			if (i >= 3 && (p[0] == 0x22 || p[0] == 0x27) && p[0] == p[i - 1]) {
				p++;
				i -= 2;
			}
			p[i] = '\0';
			if (sub_mkexe(env->progseln, p) != 0)
				continue;
		} else if (strncmp(p, "REGS", 4) == 0 && p[4] <= ' ') {
			KInt *reg = getVarPtr(env->sysReg0);
			#if (defined(KUSE64) && defined(KWINDOWS))
				printf("RAX=0x%016I64X  RCX=0x%016I64X  RDX=0x%016I64X  RBX=0x%016I64X\n", reg[ 0], reg[ 1], reg[ 2], reg[ 3]);
				printf("RFL=0x%016I64X  RSI=0x%016I64X  RDI=0x%016I64X  $DI=0x%016I64X\n", reg[-1], reg[ 6], reg[ 7], reg[-2]);
				printf("R08=0x%016I64X  R09=0x%016I64X  R10=0x%016I64X  R11=0x%016I64X\n", reg[ 8], reg[ 9], reg[10], reg[11]);
				printf("R12=0x%016I64X  R13=0x%016I64X  R14=0x%016I64X  R15=0x%016I64X\n", reg[12], reg[13], reg[14], reg[15]);
			#elif (defined(KUSE64) && defined(KPOSIX))
				printf("RAX=0x%016lX  RCX=0x%016lX  RDX=0x%016lX  RBX=0x%016lX\n", reg[ 0], reg[ 1], reg[ 2], reg[ 3]);
				printf("RFL=0x%016lX  RSI=0x%016lX  RDI=0x%016lX  $DI=0x%016lX\n", reg[-1], reg[ 6], reg[ 7], reg[-2]);
				printf("R08=0x%016lX  R09=0x%016lX  R10=0x%016lX  R11=0x%016lX\n", reg[ 8], reg[ 9], reg[10], reg[11]);
				printf("R12=0x%016lX  R13=0x%016lX  R14=0x%016lX  R15=0x%016lX\n", reg[12], reg[13], reg[14], reg[15]);
			#elif (defined(KUSE32))
				printf("EAX=0x%08X  ECX=0x%08X  EDX=0x%08X  EBX=0x%08X\n", reg[ 0], reg[ 1], reg[ 2], reg[ 3]);
				printf("EFL=0x%08X  ESI=0x%08X  EDI=0x%08X  $DI=0x%08X\n", reg[-1], reg[ 6], reg[ 7], reg[-2]);
			#endif
			printf("$LINE=%d\n", (int) *getVarPtr(env->sysLine));
			continue;
		} else if (strncmp(p, "RESUME", 6) == 0 && p[6] <= ' ') {
			return;
		} else if (strncmp(p, "LOAD", 4) == 0 && p[4] == ' ') {	// Iɂincludeɋ߂.
			KVarArea bkup;
			KVarArea_init(&bkup, &kmemPool, 0);
			KVarArea_addBytes(&bkup, env->batch.s - bati, env->batch.p + bati);
			env->batch.s = 0;
			for (p += 4; *p == ' ' || *p == '\t'; p++);
			for (i = 0; p[i] >= ' ' && p[i] != ';'; i++);
			if (i >= 3 && (p[0] == 0x22 || p[0] == 0x27) && p[0] == p[i - 1]) {
				p++;
				i -= 2;
			}
			p[i] = '\0';
			FILE *fp = fopen(p, "rb");
			if (fp == 0) {
				fprintf(stderr, "fopen error.\n");
			} else {
				for (;;) {
					KVarArea_reserve(&env->batch, env->batch.s + 65536);
					i = fread(env->batch.p + env->batch.s, 1, 65536, fp);
					if (i <= 0) break;
					env->batch.s += i;
				}
				fclose(fp);
				KVarArea_addChar(&env->batch, '\n');
			}
			KVarArea_addBytes(&env->batch, bkup.s, bkup.p);
			bati = 0;
			KVarArea_dest(&bkup);
			continue;
		} else {
			env->ln = -1;
			eval0(-1);
			if (evalAdd(strlen(p), p) != 0)
				continue;
			eval1();
		}
		if (env->mode_mkexe == 0) {
			i = *getVarPtr(env->sysLine);
			if (i >= 0)
				printf("break in %d\n", (int) i);
			if (i == -1)
				printf("break\n");
		}
		env->mode_mkexe = 0;
		eval2();
		if (env->timemode != 0 && flg == 0)
			printf("(%.3f[sec])\n", (clock() - t0) / (double) CLOCKS_PER_SEC);
	}
}

int setArg1(int argc, const char **argv, const char **arg1v)
{
	int i, arg1c = 0;
	for (i = 0; i < argc; i++) {
		if (argv[i][0] != '-') {
			arg1v[arg1c] = argv[i];
			arg1c++;
			continue;
		}
		if (strcmp(argv[i], "--consonly") == 0)
			env->mode_consonly = 1;
	}
	arg1v[arg1c] = 0;
	return arg1c;
}

int bla_main(int argc, const char **argv)
{
	const char **arg1v;
	int arg1c;
	ktestUseBit();
	KMemPool_init(&kmemPool, 0);
	env = KMemPool_allocC(&kmemPool, sizeof (Env));
	arg1v = KMemPool_alloc(&kmemPool, (argc + 1) * sizeof (char **));
	arg1c = setArg1(argc, argv, arg1v);
	KLexer_initAd(&env->lx, &kmemPool, sizeof (KLexer_Elmt), 0, 0, 0);
	env->brg0 = env->brg1 = kmallocRWX(1024 * 1024);
	env->code = env->brg2 = env->brg0 + 16 * 256;	// 4KB.
	KVarArea_initAd(&env->var, &kmemPool, 0);
	KVarArea_initAd(&env->batch, &kmemPool, 0);
	KVarArea_initAd(&env->vInfo, &kmemPool, 0);
	KStringId_init(&env->spCacheKey, &kmemPool, 1, 0, 0, 0);
	KVarArea_init(&env->spCacheVal, &kmemPool, 0);
	env->lx.sid->cbf = cb_kstringId;
	env->mode_mkexe = 0;
	KStringId_ins(env->lx.sid, "; $LINE == != < > <= >= $RETVAL CONST ( )");
	env->sysLine = KStringId_getId0(env->lx.sid, "$LINE");
	env->sysRetVal = KStringId_getId0(env->lx.sid, "$RETVAL");
	KVarArea_initAd(&env->prog, &kmemPool, 0);
	KVarArea_initAd(&env->mkexe_txt, &kmemPool, 0);
	KVarArea_initAd(&env->mkexe_idt, &kmemPool, 0);
	KVarArea_initAd(&env->mkexe_lnk, &kmemPool, 0);
	KVarArea_initAd(&env->mkexe_dll, &kmemPool, 0);
	KIndexDict0_init(&env->mkexe_vofs, &kmemPool, sizeof (KInt), 0, 0);
	sub_progsel(0);
	kxorshift32_seed((unsigned int) time(0));
	env->gwin = es_openWin(256, 256, "graphics", 0);
	env->lwait = -1;
	KInt r00 = env->sysReg0 = *env->lx.sid->n + 2, i;
	#if (defined(KUSE64))
		KStringId_ins(env->lx.sid, "$DI RFLAGS RAX RCX RDX RBX RSP RBP RSI RDI R08 R09 R10 R11 R12 R13 R14 R15");
		env->sysReg1 = r00 + 16;
		KIndexDict0_init(&env->brg, &kmemPool, sizeof (KInt), 0, 0);
	#elif (defined(KUSE32))
		KStringId_ins(env->lx.sid, "$DI EFLAGS EAX ECX EDX EBX ESP EBP ESI EDI");
		env->sysReg1 = r00 + 8;
	#endif
	env->saveRegs = &env->code[env->qc];
	for (i = 0; i < 16; i++) {
		if (i == 4 || i == 5) continue;
		#if (defined(KUSE32))
			if (i >= 8) break;
		#endif
		env->qc += kputBin86(&env->code[env->qc], "%rx1_89_%mrm_%i32", 2, (int) i, 5, (KInt) getVarOfs(r00 + i));
	}
	env->qc += kputBin86(&env->code[env->qc], "9c  58  %rx1_89_%mrm_%i32  c3", 2, 0, 5, (KInt) getVarOfs(r00 - 1));
	env->loadRegs = &env->code[env->qc];
	env->qc += kputBin86(&env->code[env->qc], "%rx1_8b_%mrm_%i32  50  9d", 2, 0, 5, (KInt) getVarOfs(r00 - 1));
	for (i = 0; i < 16; i++) {
		if (i == 4 || i == 5) continue;
		#if (defined(KUSE32))
			if (i >= 8) break;
		#endif
		env->qc += kputBin86(&env->code[env->qc], "%rx1_8b_%mrm_%i32", 2, (int) i, 5, (KInt) getVarOfs(r00 + i));
	}
	env->qc += kputBin86(&env->code[env->qc], "c3");
	env->exitForce = &env->code[env->qc];
	env->qc += kputBin86(&env->code[env->qc], "e8_%r32  e8_%r32", (KInt) env->saveRegs, (KInt) BRG(sub_end));
	env->qc += kputBin86(&env->code[env->qc], "e8_%r32  %rx1_8b_%mrm", (KInt) BRG(sub_getPR04_0), 0, R04, R00);
	env->exitProg = &env->code[env->qc];	// X^bN߂Ăׂ֔΁ACɋAĂ.
	env->qc += kputBin86(&env->code[env->qc], "e8_%r32", (KInt) BRG(sub_getPR04_0));
	env->qc += kputBin86(&env->code[env->qc], "%rx1_8b_%mrm", 0, R02, R00);	// RDX=[RAX]
	env->qc += kputBin86(&env->code[env->qc], "%rx1_3b_%mrm", 3, R04, R02);	// if (RSP == RDX) {
	env->qc += kputBin86(&env->code[env->qc], "75_02");
	env->qc += kputBin86(&env->code[env->qc], "31_%mrm", 3, R02, R02);		// RDX=0; }
	env->qc += kputBin86(&env->code[env->qc], "%rx1_89_%mrm", 0, R02, R00);	// [RAX]=RDX
	#if (defined(KUSE64))
		env->qc += kputBin86(&env->code[env->qc], "%rx1_81_c4_f8_00_00_00"); // ADD RSP,248.  ŏɒX^bNɖ߂.  RET.
		env->qc += kputBin86(&env->code[env->qc], "58  59  5a  5b  5c  5d  5e  5f  41_58  41_59");
		env->qc += kputBin86(&env->code[env->qc], "41_5a  41_5b  41_5c  41_5d  41_5e  41_5f  c3");
	#elif (defined(KUSE32))
		env->qc += kputBin86(&env->code[env->qc], "83_c4_7c  61  c3"); // ADD ESP,124.  ŏɒX^bNɖ߂.  POPAD.  RET.
	#endif
	for (i = 1; i < arg1c; i++) {
		const char *p = arg1v[i];
		int l = strlen(p);
		if (*p == 0x22 || *p == 0x27) {
			p++;
			l -= 2;
		}
		if (i == 1 && *p != ';')
			KVarArea_addBytes(&env->batch, 5, "load ");
		KVarArea_addBytes(&env->batch, l, p);
		KVarArea_addBytes(&env->batch, 1, "\n");
	}
	for (;;)
		cmdlineLoop(0);
	return 0;
}
